from typing import List

from langchain_core.messages import AnyMessage

from ....models.openai import models as openai_models
from ...common.agent_run_context import AgentRunContext


class StreamEventState:
    sequence_number: int = 0


class ResponseEventGenerator:
    started: bool = False

    def __init__(self, logger, parent):
        self.logger = logger
        self.parent = parent  # parent generator

    def try_process_message(self, message: AnyMessage, context: AgentRunContext, stream_state: StreamEventState):
        """
        Try to process the incoming message.
        Returns a tuple of (is_processed, next_processor, events)
        is_processed: bool - whether the message was processed by this generator,
                    if True, the message has been fully handled and should not be passed to other generators
                    if False, the message was not fully handled and should be passed to the next processor
        next_processor: ResponseEventGenerator - the next processor to handle subsequent messages
        events: List[ResponseStreamEvent] - list of generated events, will be sent to client
        """
        pass

    def on_start(self) -> tuple[bool, List[openai_models.ResponseStreamEvent]]:
        """
        Generate the starting events for this layer.
        Returns a tuple of (started, events)
        """
        pass

    def on_end(self) -> tuple[bool, List[openai_models.ResponseStreamEvent]]:
        """
        Generate the ending events for this layer.
        TODO: handle different end conditions, e.g. normal end, error end, etc.
        Returns a tuple of (ended, events)
        """
        pass

    def aggregate_content(self):
        """
        Aggregate the content for this layer.
        It is called by its child processor to pass up aggregated content.
        """
        pass



import json

from langchain_core import messages as lg_messages

from ...models.openai import models as openai_models


def extract_function_call(tool_call):
    """
    Extract function call details from tool_call dict.
    Returns a tuple of (name, call_id, argument).
    """
    name = tool_call.get("name")
    call_id = tool_call.get("id")
    argument = None
    arguments_raw = tool_call.get("args")
    if isinstance(arguments_raw, str):
        argument = arguments_raw
    elif isinstance(arguments_raw, dict):
        argument = json.dumps(arguments_raw)
    return name, call_id, argument


def create_item_resource(message: lg_messages.AnyMessage) -> openai_models.ItemResource:
    """
    Create an ItemResource from a LangChain message.
    """
    return openai_models.ItemResource(id=message.id, content=message.content, metadata=message.metadata)



import logging
import os
from logging import config

from .constants import Constants

default_log_config = {
    "version": 1,
    "disable_existing_loggers": False,
    "loggers": {
        "container_agents_adapter": {
            "handlers": ["console"],
            "level": "INFO",
            "propagate": False,
        },
    },
    "handlers": {
        "console": {"formatter": "std_out", "class": "logging.StreamHandler", "level": "INFO"},
    },
    "formatters": {"std_out": {"format": "%(asctime)s - %(name)s - %(levelname)s - %(message)s"}},
}


class CustomLogger(logging.Logger):
    def _log(self, level, msg, args, exc_info=None, extra=None):
        dimensions = get_dimensions()
        if extra:
            dimensions.update(extra)
        super(CustomLogger, self)._log(level, msg, args, exc_info, extra=dimensions)


def get_dimensions():
    agent_id = os.environ.get(Constants.AGENT_ID, "test_agent_id")
    return {
        "agent_id": agent_id,
    }


def get_project_endpoint():
    project_resource_id = os.environ.get(Constants.AGENT_PROJECT_RESOURCE_ID)
    if project_resource_id:
        last_part = project_resource_id.split("/")[-1]

        parts = last_part.split("@")
        if len(parts) < 2:
            print(f"invalid project resource id: {project_resource_id}")
            return None
        account = parts[0]
        project = parts[1]
        return f"https://{account}.services.ai.azure.com/api/projects/{project}"
    else:
        print("environment variable AGENT_PROJECT_RESOURCE_ID not set.")
        return None


def get_application_insights_connection_string():
    try:
        conn_str = os.environ.get(Constants.APPLICATION_INSIGHTS_CONNECTION_STRING)
        if not conn_str:
            print("environment variable APPLICATION_INSIGHTS_CONNECTION_STRING not set.")
            project_endpoint = get_project_endpoint()
            if project_endpoint:
                # try to get the project connected application insights
                from azure.ai.projects import AIProjectClient
                from azure.identity import DefaultAzureCredential

                project_client = AIProjectClient(credential=DefaultAzureCredential(), endpoint=project_endpoint)
                conn_str = project_client.telemetry.get_connection_string()
                if not conn_str:
                    print(f"no connected application insights found for project:{project_endpoint}")
        return conn_str
    except Exception as e:
        print(f"failed to get application insights with error: {e}")
        return None


class CustomDimensionsFilter(logging.Filter):
    def filter(self, record):
        # Add custom dimensions to every log record
        dimensions = get_dimensions()
        for key, value in dimensions.items():
            setattr(record, key, value)
        return True


def configure(log_config: dict = default_log_config):
    """
    Configure logging based on the provided configuration dictionary.
    The dictionary should contain the logging configuration in a format compatible with `logging.config.dictConfig`.
    """
    try:
        logging.setLoggerClass(CustomLogger)

        config.dictConfig(log_config)

        application_insights_connection_string = get_application_insights_connection_string()
        if application_insights_connection_string:
            from azure.monitor.opentelemetry.exporter import AzureMonitorLogExporter
            from opentelemetry._logs import set_logger_provider
            from opentelemetry.sdk._logs import (
                LoggerProvider,
                LoggingHandler,
            )
            from opentelemetry.sdk._logs.export import BatchLogRecordProcessor
            from opentelemetry.sdk.resources import Resource

            logger_provider = LoggerProvider(resource=Resource.create({"service.name": "agent_runtime.agent_adapter"}))
            set_logger_provider(logger_provider)

            exporter = AzureMonitorLogExporter(connection_string=application_insights_connection_string)

            logger_provider.add_log_record_processor(BatchLogRecordProcessor(exporter))
            handler = LoggingHandler(level=logging.NOTSET, logger_provider=logger_provider)

            # Add custom filter to inject dimensions
            custom_filter = CustomDimensionsFilter()
            handler.addFilter(custom_filter)

            # Only add to container_agents_adapter namespace to avoid infrastructure logs
            app_logger = logging.getLogger("container_agents_adapter")
            app_logger.setLevel(logging.DEBUG)
            app_logger.addHandler(handler)

    except Exception as e:
        print(f"Failed to configure logging: {e}")
        pass


def get_logger():
    """
    If the logger is not already configured, it will be initialized with default settings.
    """
    return logging.getLogger("container_agents_adapter")



# pylint: disable=line-too-long,useless-suppression,too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=useless-super-delegation

import datetime
from typing import Any, Dict, List, Literal, Mapping, Optional, TYPE_CHECKING, Union, overload

from ...openai.models import ItemParam, ItemResource, Tool
from .._utils.model_base import Model as _Model, rest_discriminator, rest_field
from ._enums import AgentKind

if TYPE_CHECKING:
    from .. import models as _models
    from ...openai import models as _openai_models3


class AgentDefinition(_Model):
    """AgentDefinition.

    You probably want to use the sub-classes and not this class directly. Known sub-classes are:
    CustomAgentDefinition, PromptAgentDefinition, WorkflowDefinition

    :ivar kind: Required. Known values are: "prompt_agent", "custom_agent", and "workflow".
    :vartype kind: str or ~azureaiagents.models.AgentKind
    """

    __mapping__: Dict[str, _Model] = {}
    kind: str = rest_discriminator(name="kind", visibility=["read", "create", "update", "delete", "query"])
    """Required. Known values are: \"prompt_agent\", \"custom_agent\", and \"workflow\"."""

    @overload
    def __init__(
        self,
        *,
        kind: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class AgentId(_Model):
    """AgentId.

    :ivar type: Required. Default value is "agent_id".
    :vartype type: str
    :ivar name: The name of the agent. Required.
    :vartype name: str
    :ivar version: The version identifier of the agent.
    :vartype version: str
    """

    type: Literal["agent_id"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Required. Default value is \"agent_id\"."""
    name: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The name of the agent. Required."""
    version: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The version identifier of the agent."""

    @overload
    def __init__(
        self,
        *,
        name: str,
        version: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.type: Literal["agent_id"] = "agent_id"


class AgentLabelList(_Model):
    """AgentLabelList.

    :ivar data: A list of items used to generate this response. Required.
    :vartype data: list[~azureaiagents.models.AgentLabelObject]
    :ivar object: The object type. Always 'list'. Required. Default value is "list".
    :vartype object: str
    :ivar first_id: ID of the first item in the returned page. Required.
    :vartype first_id: str
    :ivar last_id: ID of the last item in the returned page. Required.
    :vartype last_id: str
    :ivar has_more: Whether more items are available after this page. Required.
    :vartype has_more: bool
    """

    data: List["_models.AgentLabelObject"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """A list of items used to generate this response. Required."""
    object: Literal["list"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The object type. Always 'list'. Required. Default value is \"list\"."""
    first_id: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """ID of the first item in the returned page. Required."""
    last_id: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """ID of the last item in the returned page. Required."""
    has_more: bool = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Whether more items are available after this page. Required."""

    @overload
    def __init__(
        self,
        *,
        data: List["_models.AgentLabelObject"],
        first_id: str,
        last_id: str,
        has_more: bool,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.object: Literal["list"] = "list"


class AgentLabelObject(_Model):
    """An agent label object that shows the association between agent and a specific version of agent.
    Labels are used to mark specific versions of the agent for easy retrieval.

    :ivar object: The object type, which is always 'agent.label'. Required. Default value is
     "agent.label".
    :vartype object: str
    :ivar id: The unique identifier of the label. Every label update results in a new label ID.
     Required.
    :vartype id: str
    :ivar name: The name of the label. Required.
    :vartype name: str
    :ivar created_at: The Unix timestamp (seconds) when the label was created. Required.
    :vartype created_at: ~datetime.datetime
    :ivar agent_version: The version of the agent that this label is associated with. If not
     provided, then the label is not associated with any specific version.
    :vartype agent_version: str
    """

    object: Literal["agent.label"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The object type, which is always 'agent.label'. Required. Default value is \"agent.label\"."""
    id: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The unique identifier of the label. Every label update results in a new label ID. Required."""
    name: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The name of the label. Required."""
    created_at: datetime.datetime = rest_field(
        visibility=["read", "create", "update", "delete", "query"], format="unix-timestamp"
    )
    """The Unix timestamp (seconds) when the label was created. Required."""
    agent_version: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The version of the agent that this label is associated with. If not provided, then the label is
     not associated with any specific version."""

    @overload
    def __init__(
        self,
        *,
        id: str,  # pylint: disable=redefined-builtin
        name: str,
        created_at: datetime.datetime,
        agent_version: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.object: Literal["agent.label"] = "agent.label"


class AgentList(_Model):
    """AgentList.

    :ivar data: A list of items used to generate this response. Required.
    :vartype data: list[~azureaiagents.models.AgentObject]
    :ivar object: The object type. Always 'list'. Required. Default value is "list".
    :vartype object: str
    :ivar first_id: ID of the first item in the returned page. Required.
    :vartype first_id: str
    :ivar last_id: ID of the last item in the returned page. Required.
    :vartype last_id: str
    :ivar has_more: Whether more items are available after this page. Required.
    :vartype has_more: bool
    """

    data: List["_models.AgentObject"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """A list of items used to generate this response. Required."""
    object: Literal["list"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The object type. Always 'list'. Required. Default value is \"list\"."""
    first_id: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """ID of the first item in the returned page. Required."""
    last_id: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """ID of the last item in the returned page. Required."""
    has_more: bool = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Whether more items are available after this page. Required."""

    @overload
    def __init__(
        self,
        *,
        data: List["_models.AgentObject"],
        first_id: str,
        last_id: str,
        has_more: bool,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.object: Literal["list"] = "list"


class AgentObject(_Model):
    """AgentObject.

    :ivar object: The object type, which is always 'agent'. Required. Default value is "agent".
    :vartype object: str
    :ivar id: The unique identifier of the agent. Required.
    :vartype id: str
    :ivar name: The name of the agent. Required.
    :vartype name: str
    :ivar labels: The labels and their version associations for this agent. Required.
    :vartype labels: list[~azureaiagents.models.AgentLabelObject]
    """

    object: Literal["agent"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The object type, which is always 'agent'. Required. Default value is \"agent\"."""
    id: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The unique identifier of the agent. Required."""
    name: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The name of the agent. Required."""
    labels: List["_models.AgentLabelObject"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The labels and their version associations for this agent. Required."""

    @overload
    def __init__(
        self,
        *,
        id: str,  # pylint: disable=redefined-builtin
        name: str,
        labels: List["_models.AgentLabelObject"],
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.object: Literal["agent"] = "agent"


class AgentReference(_Model):
    """AgentReference.

    :ivar type: Required. Default value is "agent_reference".
    :vartype type: str
    :ivar name: The name of the agent. Required.
    :vartype name: str
    :ivar version: The version identifier of the agent.
    :vartype version: str
    :ivar label: The label that identifies the agent.
    :vartype label: str
    """

    type: Literal["agent_reference"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Required. Default value is \"agent_reference\"."""
    name: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The name of the agent. Required."""
    version: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The version identifier of the agent."""
    label: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The label that identifies the agent."""

    @overload
    def __init__(
        self,
        *,
        name: str,
        version: Optional[str] = None,
        label: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.type: Literal["agent_reference"] = "agent_reference"


class AgentVersionList(_Model):
    """AgentVersionList.

    :ivar data: A list of items used to generate this response. Required.
    :vartype data: list[~azureaiagents.models.AgentVersionObject]
    :ivar object: The object type. Always 'list'. Required. Default value is "list".
    :vartype object: str
    :ivar first_id: ID of the first item in the returned page. Required.
    :vartype first_id: str
    :ivar last_id: ID of the last item in the returned page. Required.
    :vartype last_id: str
    :ivar has_more: Whether more items are available after this page. Required.
    :vartype has_more: bool
    """

    data: List["_models.AgentVersionObject"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """A list of items used to generate this response. Required."""
    object: Literal["list"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The object type. Always 'list'. Required. Default value is \"list\"."""
    first_id: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """ID of the first item in the returned page. Required."""
    last_id: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """ID of the last item in the returned page. Required."""
    has_more: bool = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Whether more items are available after this page. Required."""

    @overload
    def __init__(
        self,
        *,
        data: List["_models.AgentVersionObject"],
        first_id: str,
        last_id: str,
        has_more: bool,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.object: Literal["list"] = "list"


class AgentVersionObject(_Model):
    """AgentVersionObject.

    :ivar object: The object type, which is always 'agent.version'. Required. Default value is
     "agent.version".
    :vartype object: str
    :ivar id: The unique identifier of the agent version. Required.
    :vartype id: str
    :ivar name: The name of the agent. Name can be used to retrieve/update/delete the agent.
     Required.
    :vartype name: str
    :ivar version: The version identifier of the agent. Agents are immutable and every update
     creates a new version while keeping the name same. Required.
    :vartype version: str
    :ivar description: A human-readable description of the agent.
    :vartype description: str
    :ivar metadata: Arbitrary key-value metadata to associate with the agent.
    :vartype metadata: dict[str, str]
    :ivar created_at: The Unix timestamp (seconds) when the agent was created. Required.
    :vartype created_at: ~datetime.datetime
    :ivar labels: The labels associated with this version of the agent. Labels are used to mark
     specific versions of the agent for easy retrieval. Required.
    :vartype labels: list[str]
    :ivar status: The status of agent. Required. Is one of the following types:
     Literal["creating"], Literal["updating"], Literal["deleting"], Literal["active"]
    :vartype status: str or str or str or str
    :ivar definition: Required.
    :vartype definition: ~azureaiagents.models.AgentDefinition
    """

    object: Literal["agent.version"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The object type, which is always 'agent.version'. Required. Default value is \"agent.version\"."""
    id: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The unique identifier of the agent version. Required."""
    name: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The name of the agent. Name can be used to retrieve/update/delete the agent. Required."""
    version: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The version identifier of the agent. Agents are immutable and every update creates a new
     version while keeping the name same. Required."""
    description: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """A human-readable description of the agent."""
    metadata: Optional[Dict[str, str]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Arbitrary key-value metadata to associate with the agent."""
    created_at: datetime.datetime = rest_field(
        visibility=["read", "create", "update", "delete", "query"], format="unix-timestamp"
    )
    """The Unix timestamp (seconds) when the agent was created. Required."""
    labels: List[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The labels associated with this version of the agent. Labels are used to mark specific versions
     of the agent for easy retrieval. Required."""
    status: Literal["creating", "updating", "deleting", "active"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """The status of agent. Required. Is one of the following types: Literal[\"creating\"],
     Literal[\"updating\"], Literal[\"deleting\"], Literal[\"active\"]"""
    definition: "_models.AgentDefinition" = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Required."""

    @overload
    def __init__(
        self,
        *,
        id: str,  # pylint: disable=redefined-builtin
        name: str,
        version: str,
        created_at: datetime.datetime,
        labels: List[str],
        status: Literal["creating", "updating", "deleting", "active"],
        definition: "_models.AgentDefinition",
        description: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.object: Literal["agent.version"] = "agent.version"


class CaptureSemanticEventsTool(Tool, discriminator="capture_semantic_events"):
    """CaptureSemanticEventsTool.

    :ivar type: The type of the tool. Always ``capture_semantic_events``. Required. Default value
     is "capture_semantic_events".
    :vartype type: str
    :ivar events: Set of structured events to catpure from the model. Required.
    :vartype events: dict[str, ~azureaiagents.models.SemanticEventDefinition]
    """

    type: Literal["capture_semantic_events"] = rest_discriminator(name="type", visibility=["read", "create", "update", "delete", "query"])  # type: ignore
    """The type of the tool. Always ``capture_semantic_events``. Required. Default value is
     \"capture_semantic_events\"."""
    events: Dict[str, "_models.SemanticEventDefinition"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Set of structured events to catpure from the model. Required."""

    @overload
    def __init__(
        self,
        *,
        events: Dict[str, "_models.SemanticEventDefinition"],
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, type="capture_semantic_events", **kwargs)


class CaptureStructuredOutputsTool(Tool, discriminator="capture_structured_outputs"):
    """CaptureStructuredOutputsTool.

    :ivar type: The type of the tool. Always ``capture_structured_outputs``. Required. Default
     value is "capture_structured_outputs".
    :vartype type: str
    :ivar outputs: Set of structured outputs to capture from the model. Required.
    :vartype outputs: dict[str, ~azureaiagents.models.StructuredOutputDefinition]
    """

    type: Literal["capture_structured_outputs"] = rest_discriminator(name="type", visibility=["read", "create", "update", "delete", "query"])  # type: ignore
    """The type of the tool. Always ``capture_structured_outputs``. Required. Default value is
     \"capture_structured_outputs\"."""
    outputs: Dict[str, "_models.StructuredOutputDefinition"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Set of structured outputs to capture from the model. Required."""

    @overload
    def __init__(
        self,
        *,
        outputs: Dict[str, "_models.StructuredOutputDefinition"],
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, type="capture_structured_outputs", **kwargs)


class ConversationItemInput(_Model):
    """ConversationItemInput.

    :ivar item: Required.
    :vartype item: ~openai.models.ItemParam
    """

    item: "_openai_models3.ItemParam" = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Required."""

    @overload
    def __init__(
        self,
        *,
        item: "_openai_models3.ItemParam",
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class ConversationItemList(_Model):
    """ConversationItemList.

    :ivar data: A list of items used to generate this response. Required.
    :vartype data: list[~openai.models.ItemResource]
    :ivar object: The object type. Always 'list'. Required. Default value is "list".
    :vartype object: str
    :ivar first_id: ID of the first item in the returned page. Required.
    :vartype first_id: str
    :ivar last_id: ID of the last item in the returned page. Required.
    :vartype last_id: str
    :ivar has_more: Whether more items are available after this page. Required.
    :vartype has_more: bool
    """

    data: List["_openai_models3.ItemResource"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """A list of items used to generate this response. Required."""
    object: Literal["list"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The object type. Always 'list'. Required. Default value is \"list\"."""
    first_id: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """ID of the first item in the returned page. Required."""
    last_id: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """ID of the last item in the returned page. Required."""
    has_more: bool = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Whether more items are available after this page. Required."""

    @overload
    def __init__(
        self,
        *,
        data: List["_openai_models3.ItemResource"],
        first_id: str,
        last_id: str,
        has_more: bool,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.object: Literal["list"] = "list"


class ConversationList(_Model):
    """ConversationList.

    :ivar data: A list of items used to generate this response. Required.
    :vartype data: list[~azureaiagents.models.ConversationObject]
    :ivar object: The object type. Always 'list'. Required. Default value is "list".
    :vartype object: str
    :ivar first_id: ID of the first item in the returned page. Required.
    :vartype first_id: str
    :ivar last_id: ID of the last item in the returned page. Required.
    :vartype last_id: str
    :ivar has_more: Whether more items are available after this page. Required.
    :vartype has_more: bool
    """

    data: List["_models.ConversationObject"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """A list of items used to generate this response. Required."""
    object: Literal["list"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The object type. Always 'list'. Required. Default value is \"list\"."""
    first_id: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """ID of the first item in the returned page. Required."""
    last_id: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """ID of the last item in the returned page. Required."""
    has_more: bool = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Whether more items are available after this page. Required."""

    @overload
    def __init__(
        self,
        *,
        data: List["_models.ConversationObject"],
        first_id: str,
        last_id: str,
        has_more: bool,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.object: Literal["list"] = "list"


class ConversationObject(_Model):
    """ConversationObject.

    :ivar object: Required. Default value is "conversation".
    :vartype object: str
    :ivar id: Required.
    :vartype id: str
    :ivar created_at: The Unix timestamp (seconds) when the conversation was created. Required.
    :vartype created_at: ~datetime.datetime
    :ivar metadata:
    :vartype metadata: dict[str, str]
    """

    object: Literal["conversation"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Required. Default value is \"conversation\"."""
    id: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Required."""
    created_at: datetime.datetime = rest_field(
        visibility=["read", "create", "update", "delete", "query"], format="unix-timestamp"
    )
    """The Unix timestamp (seconds) when the conversation was created. Required."""
    metadata: Optional[Dict[str, str]] = rest_field(visibility=["read", "create", "update", "delete", "query"])

    @overload
    def __init__(
        self,
        *,
        id: str,  # pylint: disable=redefined-builtin
        created_at: datetime.datetime,
        metadata: Optional[Dict[str, str]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.object: Literal["conversation"] = "conversation"


class ConversationOptions(_Model):
    """ConversationOptions.

    :ivar id: The conversation ID to associate with this response.
    :vartype id: str
    :ivar commit: If true, commit the outputs of the response to the conversation if a conversation
     is specified by the user. Required.
    :vartype commit: bool
    """

    id: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The conversation ID to associate with this response."""
    commit: bool = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """If true, commit the outputs of the response to the conversation if a conversation is specified
     by the user. Required."""

    @overload
    def __init__(
        self,
        *,
        commit: bool,
        id: Optional[str] = None,  # pylint: disable=redefined-builtin
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class CreateAgentVersionRequest(_Model):
    """CreateAgentVersionRequest.

    :ivar description: A human-readable description of the agent.
    :vartype description: str
    :ivar metadata: Arbitrary key-value metadata to associate with the agent.
    :vartype metadata: dict[str, str]
    :ivar definition: The agent definition. This can be a workflow, custom agent, or a simple agent
     definition. Required.
    :vartype definition: ~azureaiagents.models.AgentDefinition
    """

    description: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """A human-readable description of the agent."""
    metadata: Optional[Dict[str, str]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Arbitrary key-value metadata to associate with the agent."""
    definition: "_models.AgentDefinition" = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The agent definition. This can be a workflow, custom agent, or a simple agent definition.
     Required."""

    @overload
    def __init__(
        self,
        *,
        definition: "_models.AgentDefinition",
        description: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class CreateConversationInput(_Model):
    """CreateConversationInput.

    :ivar metadata:
    :vartype metadata: dict[str, str]
    :ivar items_property:
    :vartype items_property: list[~openai.models.ItemParam]
    """

    metadata: Optional[Dict[str, str]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    items_property: Optional[List["_openai_models3.ItemParam"]] = rest_field(
        name="items", visibility=["read", "create", "update", "delete", "query"]
    )

    @overload
    def __init__(
        self,
        *,
        metadata: Optional[Dict[str, str]] = None,
        items_property: Optional[List["_openai_models3.ItemParam"]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class CreateResponse(_Model):
    """CreateResponse.

    :ivar agent: The agent to use for generating the response. Required.
    :vartype agent: ~azureaiagents.models.AgentReference
    :ivar conversation: The conversation options to associate with this response.
    :vartype conversation: ~azureaiagents.models.ConversationOptions
    :ivar metadata: Set of 16 key-value pairs that can be attached to an object. This can be
     useful for storing additional information about the object in a structured
     format, and querying for objects via API or the dashboard.

     Keys are strings with a maximum length of 64 characters. Values are strings
     with a maximum length of 512 characters.
    :vartype metadata: dict[str, str]
    :ivar temperature: What sampling temperature to use, between 0 and 2. Higher values like 0.8
     will make the output more random, while lower values like 0.2 will make it more focused and
     deterministic.
     We generally recommend altering this or ``top_p`` but not both.
    :vartype temperature: float
    :ivar top_p: An alternative to sampling with temperature, called nucleus sampling,
     where the model considers the results of the tokens with top_p probability
     mass. So 0.1 means only the tokens comprising the top 10% probability mass
     are considered.

     We generally recommend altering this or ``temperature`` but not both.
    :vartype top_p: float
    :ivar user: A unique identifier representing your end-user, which can help OpenAI to monitor
     and detect abuse. `Learn more </docs/guides/safety-best-practices#end-user-ids>`_.
    :vartype user: str
    :ivar service_tier: Known values are: "auto", "default", "flex", "scale", and "priority".
    :vartype service_tier: str or ~openai.models.ServiceTier
    :ivar top_logprobs: An integer between 0 and 20 specifying the number of most likely tokens to
     return at each token position, each with an associated log probability.
    :vartype top_logprobs: int
    :ivar previous_response_id: The unique ID of the previous response to the model. Use this to
     create multi-turn conversations. Learn more about
     `conversation state </docs/guides/conversation-state>`_.
    :vartype previous_response_id: str
    :ivar model: Model ID used to generate the response, like ``gpt-4o`` or ``o3``. OpenAI
     offers a wide range of models with different capabilities, performance
     characteristics, and price points. Refer to the `model guide </docs/models>`_
     to browse and compare available models. Known values are: "gpt-4.1", "gpt-4.1-mini",
     "gpt-4.1-nano", "gpt-4.1-2025-04-14", "gpt-4.1-mini-2025-04-14", "gpt-4.1-nano-2025-04-14",
     "o4-mini", "o4-mini-2025-04-16", "o3", "o3-2025-04-16", "o3-mini", "o3-mini-2025-01-31", "o1",
     "o1-2024-12-17", "o1-preview", "o1-preview-2024-09-12", "o1-mini", "o1-mini-2024-09-12",
     "gpt-4o", "gpt-4o-2024-11-20", "gpt-4o-2024-08-06", "gpt-4o-2024-05-13",
     "gpt-4o-audio-preview", "gpt-4o-audio-preview-2024-10-01", "gpt-4o-audio-preview-2024-12-17",
     "gpt-4o-audio-preview-2025-06-03", "gpt-4o-mini-audio-preview",
     "gpt-4o-mini-audio-preview-2024-12-17", "gpt-4o-search-preview", "gpt-4o-mini-search-preview",
     "gpt-4o-search-preview-2025-03-11", "gpt-4o-mini-search-preview-2025-03-11",
     "chatgpt-4o-latest", "codex-mini-latest", "gpt-4o-mini", "gpt-4o-mini-2024-07-18",
     "gpt-4-turbo", "gpt-4-turbo-2024-04-09", "gpt-4-0125-preview", "gpt-4-turbo-preview",
     "gpt-4-1106-preview", "gpt-4-vision-preview", "gpt-4", "gpt-4-0314", "gpt-4-0613", "gpt-4-32k",
     "gpt-4-32k-0314", "gpt-4-32k-0613", "gpt-3.5-turbo", "gpt-3.5-turbo-16k", "gpt-3.5-turbo-0301",
     "gpt-3.5-turbo-0613", "gpt-3.5-turbo-1106", "gpt-3.5-turbo-0125", "gpt-3.5-turbo-16k-0613",
     "o1-pro", "o1-pro-2025-03-19", "o3-pro", "o3-pro-2025-06-10", "o3-deep-research",
     "o3-deep-research-2025-06-26", "o4-mini-deep-research", "o4-mini-deep-research-2025-06-26",
     "computer-use-preview", and "computer-use-preview-2025-03-11".
    :vartype model: str or ~openai.models.ModelIdsResponses
    :ivar reasoning:
    :vartype reasoning: ~openai.models.Reasoning
    :ivar background: Whether to run the model response in the background.
     `Learn more </docs/guides/background>`_.
    :vartype background: bool
    :ivar max_output_tokens: An upper bound for the number of tokens that can be generated for a
     response, including visible output tokens and `reasoning tokens </docs/guides/reasoning>`_.
    :vartype max_output_tokens: int
    :ivar max_tool_calls: The maximum number of total calls to built-in tools that can be processed
     in a response. This maximum number applies across all built-in tool calls, not per individual
     tool. Any further attempts to call a tool by the model will be ignored.
    :vartype max_tool_calls: int
    :ivar text: Configuration options for a text response from the model. Can be plain
     text or structured JSON data. Learn more:

     * [Text inputs and outputs](/docs/guides/text)
     * [Structured Outputs](/docs/guides/structured-outputs).
    :vartype text: ~openai.models.CreateResponseText
    :ivar tools: An array of tools the model may call while generating a response. You
     can specify which tool to use by setting the ``tool_choice`` parameter.

     The two categories of tools you can provide the model are:



     * **Built-in tools**: Tools that are provided by OpenAI that extend the
     model's capabilities, like [web search](/docs/guides/tools-web-search)
     or [file search](/docs/guides/tools-file-search). Learn more about
     [built-in tools](/docs/guides/tools).
     * **Function calls (custom tools)**: Functions that are defined by you,
     enabling the model to call your own code. Learn more about
     [function calling](/docs/guides/function-calling).
    :vartype tools: list[~openai.models.Tool]
    :ivar tool_choice: How the model should select which tool (or tools) to use when generating
     a response. See the ``tools`` parameter to see how to specify which tools
     the model can call. Is either a Union[str, "_openai_models2.ToolChoiceOptions"] type or a
     ToolChoiceObject type.
    :vartype tool_choice: str or ~openai.models.ToolChoiceOptions or
     ~openai.models.ToolChoiceObject
    :ivar prompt:
    :vartype prompt: ~openai.models.Prompt
    :ivar truncation: The truncation strategy to use for the model response.

     * `auto`: If the context of this response and previous ones exceeds
     the model's context window size, the model will truncate the
     response to fit the context window by dropping input items in the
     middle of the conversation.
     * `disabled` (default): If a model response will exceed the context window
     size for a model, the request will fail with a 400 error. Is either a Literal["auto"] type or a
     Literal["disabled"] type.
    :vartype truncation: str or str
    :ivar input: Text, image, or file inputs to the model, used to generate a response.

     Learn more:

     * [Text inputs and outputs](/docs/guides/text)
     * [Image inputs](/docs/guides/images)
     * [File inputs](/docs/guides/pdf-files)
     * [Conversation state](/docs/guides/conversation-state)
     * [Function calling](/docs/guides/function-calling). Is either a str type or a
     [Union["_openai_models2.ImplicitUserMessage", "_openai_models2.ItemParam"]] type.
    :vartype input: str or list[~openai.models.ImplicitUserMessage or ~openai.models.ItemParam]
    :ivar include: Specify additional output data to include in the model response. Currently
     supported values are:

     * `code_interpreter_call.outputs`: Includes the outputs of python code execution
     in code interpreter tool call items.
     * `computer_call_output.output.image_url`: Include image urls from the computer call output.
     * `file_search_call.results`: Include the search results of
     the file search tool call.
     * `message.input_image.image_url`: Include image urls from the input message.
     * `message.output_text.logprobs`: Include logprobs with assistant messages.
     * `reasoning.encrypted_content`: Includes an encrypted version of reasoning
     tokens in reasoning item outputs. This enables reasoning items to be used in
     multi-turn conversations when using the Responses API statelessly (like
     when the `store` parameter is set to `false`, or when an organization is
     enrolled in the zero data retention program).
    :vartype include: list[str or ~openai.models.Includable]
    :ivar parallel_tool_calls: Whether to allow the model to run tool calls in parallel.
    :vartype parallel_tool_calls: bool
    :ivar store: Whether to store the generated model response for later retrieval via
     API.
    :vartype store: bool
    :ivar instructions: A system (or developer) message inserted into the model's context.

     When using along with ``previous_response_id``, the instructions from a previous
     response will not be carried over to the next response. This makes it simple
     to swap out system (or developer) messages in new responses.
    :vartype instructions: str
    :ivar stream: If set to true, the model response data will be streamed to the client
     as it is generated using `server-sent events
     <https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#Event_stream_format>`_.
     See the `Streaming section below </docs/api-reference/responses-streaming>`_
     for more information.
    :vartype stream: bool
    """

    agent: "_models.AgentReference" = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The agent to use for generating the response. Required."""
    conversation: Optional["_models.ConversationOptions"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """The conversation options to associate with this response."""
    metadata: Optional[Dict[str, str]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Set of 16 key-value pairs that can be attached to an object. This can be
     useful for storing additional information about the object in a structured
     format, and querying for objects via API or the dashboard.
     
     Keys are strings with a maximum length of 64 characters. Values are strings
     with a maximum length of 512 characters."""
    temperature: Optional[float] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """What sampling temperature to use, between 0 and 2. Higher values like 0.8 will make the output
     more random, while lower values like 0.2 will make it more focused and deterministic.
     We generally recommend altering this or ``top_p`` but not both."""
    top_p: Optional[float] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """An alternative to sampling with temperature, called nucleus sampling,
     where the model considers the results of the tokens with top_p probability
     mass. So 0.1 means only the tokens comprising the top 10% probability mass
     are considered.
     
     We generally recommend altering this or ``temperature`` but not both."""
    user: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """A unique identifier representing your end-user, which can help OpenAI to monitor and detect
     abuse. `Learn more </docs/guides/safety-best-practices#end-user-ids>`_."""
    service_tier: Optional[Union[str, "_openai_models3.ServiceTier"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Known values are: \"auto\", \"default\", \"flex\", \"scale\", and \"priority\"."""
    top_logprobs: Optional[int] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """An integer between 0 and 20 specifying the number of most likely tokens to return at each token
     position, each with an associated log probability."""
    previous_response_id: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The unique ID of the previous response to the model. Use this to
     create multi-turn conversations. Learn more about
     `conversation state </docs/guides/conversation-state>`_."""
    model: Optional[Union[str, "_openai_models3.ModelIdsResponses"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Model ID used to generate the response, like ``gpt-4o`` or ``o3``. OpenAI
     offers a wide range of models with different capabilities, performance
     characteristics, and price points. Refer to the `model guide </docs/models>`_
     to browse and compare available models. Known values are: \"gpt-4.1\", \"gpt-4.1-mini\",
     \"gpt-4.1-nano\", \"gpt-4.1-2025-04-14\", \"gpt-4.1-mini-2025-04-14\",
     \"gpt-4.1-nano-2025-04-14\", \"o4-mini\", \"o4-mini-2025-04-16\", \"o3\", \"o3-2025-04-16\",
     \"o3-mini\", \"o3-mini-2025-01-31\", \"o1\", \"o1-2024-12-17\", \"o1-preview\",
     \"o1-preview-2024-09-12\", \"o1-mini\", \"o1-mini-2024-09-12\", \"gpt-4o\",
     \"gpt-4o-2024-11-20\", \"gpt-4o-2024-08-06\", \"gpt-4o-2024-05-13\", \"gpt-4o-audio-preview\",
     \"gpt-4o-audio-preview-2024-10-01\", \"gpt-4o-audio-preview-2024-12-17\",
     \"gpt-4o-audio-preview-2025-06-03\", \"gpt-4o-mini-audio-preview\",
     \"gpt-4o-mini-audio-preview-2024-12-17\", \"gpt-4o-search-preview\",
     \"gpt-4o-mini-search-preview\", \"gpt-4o-search-preview-2025-03-11\",
     \"gpt-4o-mini-search-preview-2025-03-11\", \"chatgpt-4o-latest\", \"codex-mini-latest\",
     \"gpt-4o-mini\", \"gpt-4o-mini-2024-07-18\", \"gpt-4-turbo\", \"gpt-4-turbo-2024-04-09\",
     \"gpt-4-0125-preview\", \"gpt-4-turbo-preview\", \"gpt-4-1106-preview\",
     \"gpt-4-vision-preview\", \"gpt-4\", \"gpt-4-0314\", \"gpt-4-0613\", \"gpt-4-32k\",
     \"gpt-4-32k-0314\", \"gpt-4-32k-0613\", \"gpt-3.5-turbo\", \"gpt-3.5-turbo-16k\",
     \"gpt-3.5-turbo-0301\", \"gpt-3.5-turbo-0613\", \"gpt-3.5-turbo-1106\", \"gpt-3.5-turbo-0125\",
     \"gpt-3.5-turbo-16k-0613\", \"o1-pro\", \"o1-pro-2025-03-19\", \"o3-pro\",
     \"o3-pro-2025-06-10\", \"o3-deep-research\", \"o3-deep-research-2025-06-26\",
     \"o4-mini-deep-research\", \"o4-mini-deep-research-2025-06-26\", \"computer-use-preview\", and
     \"computer-use-preview-2025-03-11\"."""
    reasoning: Optional["_openai_models3.Reasoning"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    background: Optional[bool] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Whether to run the model response in the background.
     `Learn more </docs/guides/background>`_."""
    max_output_tokens: Optional[int] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """An upper bound for the number of tokens that can be generated for a response, including visible
     output tokens and `reasoning tokens </docs/guides/reasoning>`_."""
    max_tool_calls: Optional[int] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The maximum number of total calls to built-in tools that can be processed in a response. This
     maximum number applies across all built-in tool calls, not per individual tool. Any further
     attempts to call a tool by the model will be ignored."""
    text: Optional["_openai_models3.CreateResponseText"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Configuration options for a text response from the model. Can be plain
     text or structured JSON data. Learn more:
     
     * [Text inputs and outputs](/docs/guides/text)
     * [Structured Outputs](/docs/guides/structured-outputs)."""
    tools: Optional[List["_openai_models3.Tool"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """An array of tools the model may call while generating a response. You
     can specify which tool to use by setting the ``tool_choice`` parameter.
     
     The two categories of tools you can provide the model are:
     
     
     
     * **Built-in tools**: Tools that are provided by OpenAI that extend the
     model's capabilities, like [web search](/docs/guides/tools-web-search)
     or [file search](/docs/guides/tools-file-search). Learn more about
     [built-in tools](/docs/guides/tools).
     * **Function calls (custom tools)**: Functions that are defined by you,
     enabling the model to call your own code. Learn more about
     [function calling](/docs/guides/function-calling)."""
    tool_choice: Optional[Union[str, "_openai_models3.ToolChoiceOptions", "_openai_models3.ToolChoiceObject"]] = (
        rest_field(visibility=["read", "create", "update", "delete", "query"])
    )
    """How the model should select which tool (or tools) to use when generating
     a response. See the ``tools`` parameter to see how to specify which tools
     the model can call. Is either a Union[str, \"_openai_models2.ToolChoiceOptions\"] type or a
     ToolChoiceObject type."""
    prompt: Optional["_openai_models3.Prompt"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    truncation: Optional[Literal["auto", "disabled"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """The truncation strategy to use for the model response.
     
     * `auto`: If the context of this response and previous ones exceeds
     the model's context window size, the model will truncate the
     response to fit the context window by dropping input items in the
     middle of the conversation.
     * `disabled` (default): If a model response will exceed the context window
     size for a model, the request will fail with a 400 error. Is either a Literal[\"auto\"] type or
     a Literal[\"disabled\"] type."""
    input: Optional[Union[str, List[Union["_openai_models3.ImplicitUserMessage", "_openai_models3.ItemParam"]]]] = (
        rest_field(visibility=["read", "create", "update", "delete", "query"])
    )
    """Text, image, or file inputs to the model, used to generate a response.
     
     Learn more:
     
     * [Text inputs and outputs](/docs/guides/text)
     * [Image inputs](/docs/guides/images)
     * [File inputs](/docs/guides/pdf-files)
     * [Conversation state](/docs/guides/conversation-state)
     * [Function calling](/docs/guides/function-calling). Is either a str type or a
     [Union[\"_openai_models2.ImplicitUserMessage\", \"_openai_models2.ItemParam\"]] type."""
    include: Optional[List[Union[str, "_openai_models3.Includable"]]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Specify additional output data to include in the model response. Currently
     supported values are:
     
     * `code_interpreter_call.outputs`: Includes the outputs of python code execution
     in code interpreter tool call items.
     * `computer_call_output.output.image_url`: Include image urls from the computer call output.
     * `file_search_call.results`: Include the search results of
     the file search tool call.
     * `message.input_image.image_url`: Include image urls from the input message.
     * `message.output_text.logprobs`: Include logprobs with assistant messages.
     * `reasoning.encrypted_content`: Includes an encrypted version of reasoning
     tokens in reasoning item outputs. This enables reasoning items to be used in
     multi-turn conversations when using the Responses API statelessly (like
     when the `store` parameter is set to `false`, or when an organization is
     enrolled in the zero data retention program)."""
    parallel_tool_calls: Optional[bool] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Whether to allow the model to run tool calls in parallel."""
    store: Optional[bool] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Whether to store the generated model response for later retrieval via
     API."""
    instructions: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """A system (or developer) message inserted into the model's context.
     
     When using along with ``previous_response_id``, the instructions from a previous
     response will not be carried over to the next response. This makes it simple
     to swap out system (or developer) messages in new responses."""
    stream: Optional[bool] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """If set to true, the model response data will be streamed to the client
     as it is generated using `server-sent events
     <https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#Event_stream_format>`_.
     See the `Streaming section below </docs/api-reference/responses-streaming>`_
     for more information."""

    @overload
    def __init__(  # pylint: disable=too-many-locals
        self,
        *,
        agent: "_models.AgentReference",
        conversation: Optional["_models.ConversationOptions"] = None,
        metadata: Optional[Dict[str, str]] = None,
        temperature: Optional[float] = None,
        top_p: Optional[float] = None,
        user: Optional[str] = None,
        service_tier: Optional[Union[str, "_openai_models3.ServiceTier"]] = None,
        top_logprobs: Optional[int] = None,
        previous_response_id: Optional[str] = None,
        model: Optional[Union[str, "_openai_models3.ModelIdsResponses"]] = None,
        reasoning: Optional["_openai_models3.Reasoning"] = None,
        background: Optional[bool] = None,
        max_output_tokens: Optional[int] = None,
        max_tool_calls: Optional[int] = None,
        text: Optional["_openai_models3.CreateResponseText"] = None,
        tools: Optional[List["_openai_models3.Tool"]] = None,
        tool_choice: Optional[
            Union[str, "_openai_models3.ToolChoiceOptions", "_openai_models3.ToolChoiceObject"]
        ] = None,
        prompt: Optional["_openai_models3.Prompt"] = None,
        truncation: Optional[Literal["auto", "disabled"]] = None,
        input: Optional[
            Union[str, List[Union["_openai_models3.ImplicitUserMessage", "_openai_models3.ItemParam"]]]
        ] = None,
        include: Optional[List[Union[str, "_openai_models3.Includable"]]] = None,
        parallel_tool_calls: Optional[bool] = None,
        store: Optional[bool] = None,
        instructions: Optional[str] = None,
        stream: Optional[bool] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class CustomAgentDefinition(AgentDefinition, discriminator="custom_agent"):
    """The custom agent definition.

    :ivar kind: Required.
    :vartype kind: str or ~azureaiagents.models.CUSTOM_AGENT
    """

    kind: Literal[AgentKind.CUSTOM_AGENT] = rest_discriminator(name="kind", visibility=["read", "create", "update", "delete", "query"])  # type: ignore
    """Required."""

    @overload
    def __init__(
        self,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=AgentKind.CUSTOM_AGENT, **kwargs)


class DeleteAgentResponse(_Model):
    """A deleted agent Object.

    :ivar object: The object type. Always 'agent.deleted'. Required. Default value is
     "agent.deleted".
    :vartype object: str
    :ivar name: The name of the agent. Required.
    :vartype name: str
    :ivar deleted: Whether the agent was successfully deleted. Required.
    :vartype deleted: bool
    """

    object: Literal["agent.deleted"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The object type. Always 'agent.deleted'. Required. Default value is \"agent.deleted\"."""
    name: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The name of the agent. Required."""
    deleted: bool = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Whether the agent was successfully deleted. Required."""

    @overload
    def __init__(
        self,
        *,
        name: str,
        deleted: bool,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.object: Literal["agent.deleted"] = "agent.deleted"


class DeleteAgentVersionResponse(_Model):
    """A deleted agent version Object.

    :ivar object: The object type. Always 'agent.deleted'. Required. Default value is
     "agent.version.deleted".
    :vartype object: str
    :ivar name: The name of the agent. Required.
    :vartype name: str
    :ivar version: The version identifier of the agent. Required.
    :vartype version: str
    :ivar deleted: Whether the agent was successfully deleted. Required.
    :vartype deleted: bool
    """

    object: Literal["agent.version.deleted"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The object type. Always 'agent.deleted'. Required. Default value is \"agent.version.deleted\"."""
    name: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The name of the agent. Required."""
    version: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The version identifier of the agent. Required."""
    deleted: bool = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Whether the agent was successfully deleted. Required."""

    @overload
    def __init__(
        self,
        *,
        name: str,
        version: str,
        deleted: bool,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.object: Literal["agent.version.deleted"] = "agent.version.deleted"


class DeleteConversationItemResponse(_Model):
    """A deleted conversation item Object.

    :ivar object: The object type. Always 'conversation.item.deleted'. Required. Default value is
     "conversation.item.deleted".
    :vartype object: str
    :ivar id: The id of the conversation item. Required.
    :vartype id: str
    :ivar deleted: Whether the conversation item was successfully deleted. Required.
    :vartype deleted: bool
    """

    object: Literal["conversation.item.deleted"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """The object type. Always 'conversation.item.deleted'. Required. Default value is
     \"conversation.item.deleted\"."""
    id: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The id of the conversation item. Required."""
    deleted: bool = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Whether the conversation item was successfully deleted. Required."""

    @overload
    def __init__(
        self,
        *,
        id: str,  # pylint: disable=redefined-builtin
        deleted: bool,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.object: Literal["conversation.item.deleted"] = "conversation.item.deleted"


class DeleteConversationResponse(_Model):
    """A deleted conversation Object.

    :ivar object: The object type. Always 'conversation.deleted'. Required. Default value is
     "conversation.deleted".
    :vartype object: str
    :ivar id: The id of the conversation. Required.
    :vartype id: str
    :ivar deleted: Whether the conversation was successfully deleted. Required.
    :vartype deleted: bool
    """

    object: Literal["conversation.deleted"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The object type. Always 'conversation.deleted'. Required. Default value is
     \"conversation.deleted\"."""
    id: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The id of the conversation. Required."""
    deleted: bool = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Whether the conversation was successfully deleted. Required."""

    @overload
    def __init__(
        self,
        *,
        id: str,  # pylint: disable=redefined-builtin
        deleted: bool,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.object: Literal["conversation.deleted"] = "conversation.deleted"


class DeleteResponseResponse(_Model):
    """DeleteResponseResponse.

    :ivar id: Required.
    :vartype id: str
    :ivar object: Required. Default value is "response".
    :vartype object: str
    :ivar deleted: Required. Default value is True.
    :vartype deleted: bool
    """

    id: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Required."""
    object: Literal["response"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Required. Default value is \"response\"."""
    deleted: Literal[True] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Required. Default value is True."""

    @overload
    def __init__(
        self,
        *,
        id: str,  # pylint: disable=redefined-builtin
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.object: Literal["response"] = "response"
        self.deleted: Literal[True] = True


class Error(_Model):
    """Error.

    :ivar code: Required.
    :vartype code: str
    :ivar message: Required.
    :vartype message: str
    :ivar details:
    :vartype details: str
    :ivar errors:
    :vartype errors: dict[str, list[str]]
    """

    code: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Required."""
    message: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Required."""
    details: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    errors: Optional[Dict[str, List[str]]] = rest_field(visibility=["read", "create", "update", "delete", "query"])

    @overload
    def __init__(
        self,
        *,
        code: str,
        message: str,
        details: Optional[str] = None,
        errors: Optional[Dict[str, List[str]]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class WorkflowActionOutputItemResource(ItemResource, discriminator="workflow_action"):
    """WorkflowActionOutputItemResource.

    You probably want to use the sub-classes and not this class directly. Known sub-classes are:
    InvokeAzureAgentWorkflowActionOutputItemResource

    :ivar type: Required. Default value is "workflow_action".
    :vartype type: str
    :ivar kind: The kind of CPSDL action (e.g., 'SetVariable', 'InvokeAzureAgent'). Required.
     Default value is None.
    :vartype kind: str
    :ivar id: Unique identifier for the action. Required.
    :vartype id: str
    :ivar parent_action_id: ID of the parent action if this is a nested action.
    :vartype parent_action_id: str
    :ivar previous_action_id: ID of the previous action if this action follows another.
    :vartype previous_action_id: str
    :ivar status: Status of the action (e.g., 'in_progress', 'completed', 'failed', 'cancelled').
     Required. Is one of the following types: Literal["completed"], Literal["failed"],
     Literal["in_progress"], Literal["cancelled"]
    :vartype status: str or str or str or str
    """

    __mapping__: Dict[str, _Model] = {}
    type: Literal["workflow_action"] = rest_discriminator(name="type", visibility=["read", "create", "update", "delete", "query"])  # type: ignore
    """Required. Default value is \"workflow_action\"."""
    kind: str = rest_discriminator(name="kind", visibility=["read", "create", "update", "delete", "query"])
    """The kind of CPSDL action (e.g., 'SetVariable', 'InvokeAzureAgent'). Required. Default value is
     None."""
    parent_action_id: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """ID of the parent action if this is a nested action."""
    previous_action_id: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """ID of the previous action if this action follows another."""
    status: Literal["completed", "failed", "in_progress", "cancelled"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Status of the action (e.g., 'in_progress', 'completed', 'failed', 'cancelled'). Required. Is
     one of the following types: Literal[\"completed\"], Literal[\"failed\"],
     Literal[\"in_progress\"], Literal[\"cancelled\"]"""

    @overload
    def __init__(
        self,
        *,
        kind: str,
        id: str,  # pylint: disable=redefined-builtin
        status: Literal["completed", "failed", "in_progress", "cancelled"],
        parent_action_id: Optional[str] = None,
        previous_action_id: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, type="workflow_action", **kwargs)


class InvokeAzureAgentWorkflowActionOutputItemResource(
    WorkflowActionOutputItemResource, discriminator="InvokeAzureAgent"
):  # pylint: disable=name-too-long
    """Details about an agent invocation as part of a workflow action.

    :ivar type: Required. Default value is "workflow_action".
    :vartype type: str
    :ivar id: Unique identifier for the action. Required.
    :vartype id: str
    :ivar parent_action_id: ID of the parent action if this is a nested action.
    :vartype parent_action_id: str
    :ivar previous_action_id: ID of the previous action if this action follows another.
    :vartype previous_action_id: str
    :ivar status: Status of the action (e.g., 'in_progress', 'completed', 'failed', 'cancelled').
     Required. Is one of the following types: Literal["completed"], Literal["failed"],
     Literal["in_progress"], Literal["cancelled"]
    :vartype status: str or str or str or str
    :ivar kind: Required. Default value is "InvokeAzureAgent".
    :vartype kind: str
    :ivar agent: Agent id. Required.
    :vartype agent: ~azureaiagents.models.AgentId
    :ivar conversation_id: ID of the conversation for the agent invocation.
    :vartype conversation_id: str
    :ivar response_id: The response id for the agent invocation. Required.
    :vartype response_id: str
    """

    __mapping__: Dict[str, _Model] = {}
    kind: Literal["InvokeAzureAgent"] = rest_discriminator(name="kind", visibility=["read", "create", "update", "delete", "query"])  # type: ignore
    """Required. Default value is \"InvokeAzureAgent\"."""
    agent: "_models.AgentId" = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Agent id. Required."""
    conversation_id: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """ID of the conversation for the agent invocation."""
    response_id: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The response id for the agent invocation. Required."""

    @overload
    def __init__(
        self,
        *,
        id: str,  # pylint: disable=redefined-builtin
        status: Literal["completed", "failed", "in_progress", "cancelled"],
        agent: "_models.AgentId",
        response_id: str,
        parent_action_id: Optional[str] = None,
        previous_action_id: Optional[str] = None,
        conversation_id: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind="InvokeAzureAgent", **kwargs)


class PromptAgentDefinition(AgentDefinition, discriminator="prompt_agent"):
    """The prompt agent definition.

    :ivar kind: Required.
    :vartype kind: str or ~azureaiagents.models.PROMPT_AGENT
    :ivar model: Model ID used to generate the response, like ``gpt-4o`` or ``o3``. OpenAI
     offers a wide range of models with different capabilities, performance
     characteristics, and price points. Refer to the `model guide </docs/models>`_
     to browse and compare available models. Known values are: "gpt-4.1", "gpt-4.1-mini",
     "gpt-4.1-nano", "gpt-4.1-2025-04-14", "gpt-4.1-mini-2025-04-14", "gpt-4.1-nano-2025-04-14",
     "o4-mini", "o4-mini-2025-04-16", "o3", "o3-2025-04-16", "o3-mini", "o3-mini-2025-01-31", "o1",
     "o1-2024-12-17", "o1-preview", "o1-preview-2024-09-12", "o1-mini", "o1-mini-2024-09-12",
     "gpt-4o", "gpt-4o-2024-11-20", "gpt-4o-2024-08-06", "gpt-4o-2024-05-13",
     "gpt-4o-audio-preview", "gpt-4o-audio-preview-2024-10-01", "gpt-4o-audio-preview-2024-12-17",
     "gpt-4o-audio-preview-2025-06-03", "gpt-4o-mini-audio-preview",
     "gpt-4o-mini-audio-preview-2024-12-17", "gpt-4o-search-preview", "gpt-4o-mini-search-preview",
     "gpt-4o-search-preview-2025-03-11", "gpt-4o-mini-search-preview-2025-03-11",
     "chatgpt-4o-latest", "codex-mini-latest", "gpt-4o-mini", "gpt-4o-mini-2024-07-18",
     "gpt-4-turbo", "gpt-4-turbo-2024-04-09", "gpt-4-0125-preview", "gpt-4-turbo-preview",
     "gpt-4-1106-preview", "gpt-4-vision-preview", "gpt-4", "gpt-4-0314", "gpt-4-0613", "gpt-4-32k",
     "gpt-4-32k-0314", "gpt-4-32k-0613", "gpt-3.5-turbo", "gpt-3.5-turbo-16k", "gpt-3.5-turbo-0301",
     "gpt-3.5-turbo-0613", "gpt-3.5-turbo-1106", "gpt-3.5-turbo-0125", "gpt-3.5-turbo-16k-0613",
     "o1-pro", "o1-pro-2025-03-19", "o3-pro", "o3-pro-2025-06-10", "o3-deep-research",
     "o3-deep-research-2025-06-26", "o4-mini-deep-research", "o4-mini-deep-research-2025-06-26",
     "computer-use-preview", and "computer-use-preview-2025-03-11".
    :vartype model: str or ~openai.models.ModelIdsResponses
    :ivar instructions: A system (or developer) message inserted into the model's context.
    :vartype instructions: str
    :ivar temperature: What sampling temperature to use, between 0 and 2. Higher values like 0.8
     will make the output more random, while lower values like 0.2 will make it more focused and
     deterministic.
     We generally recommend altering this or ``top_p`` but not both.
    :vartype temperature: float
    :ivar top_p: An alternative to sampling with temperature, called nucleus sampling,
     where the model considers the results of the tokens with top_p probability
     mass. So 0.1 means only the tokens comprising the top 10% probability mass
     are considered.

     We generally recommend altering this or ``temperature`` but not both.
    :vartype top_p: float
    :ivar reasoning:
    :vartype reasoning: ~openai.models.Reasoning
    :ivar tools: An array of tools the model may call while generating a response. You
     can specify which tool to use by setting the ``tool_choice`` parameter.
    :vartype tools: list[~openai.models.Tool]
    :ivar text: Configuration options for a text response from the model. Can be plain text or
     structured JSON data.
    :vartype text: ~azureaiagents.models.PromptAgentDefinitionText
    :ivar structured_inputs: Set of structured inputs that can participate in prompt template
     substitution or tool argument bindings.
    :vartype structured_inputs: dict[str, ~azureaiagents.models.StructuredInputDefinition]
    """

    kind: Literal[AgentKind.PROMPT_AGENT] = rest_discriminator(name="kind", visibility=["read", "create", "update", "delete", "query"])  # type: ignore
    """Required."""
    model: Optional[Union[str, "_openai_models3.ModelIdsResponses"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Model ID used to generate the response, like ``gpt-4o`` or ``o3``. OpenAI
     offers a wide range of models with different capabilities, performance
     characteristics, and price points. Refer to the `model guide </docs/models>`_
     to browse and compare available models. Known values are: \"gpt-4.1\", \"gpt-4.1-mini\",
     \"gpt-4.1-nano\", \"gpt-4.1-2025-04-14\", \"gpt-4.1-mini-2025-04-14\",
     \"gpt-4.1-nano-2025-04-14\", \"o4-mini\", \"o4-mini-2025-04-16\", \"o3\", \"o3-2025-04-16\",
     \"o3-mini\", \"o3-mini-2025-01-31\", \"o1\", \"o1-2024-12-17\", \"o1-preview\",
     \"o1-preview-2024-09-12\", \"o1-mini\", \"o1-mini-2024-09-12\", \"gpt-4o\",
     \"gpt-4o-2024-11-20\", \"gpt-4o-2024-08-06\", \"gpt-4o-2024-05-13\", \"gpt-4o-audio-preview\",
     \"gpt-4o-audio-preview-2024-10-01\", \"gpt-4o-audio-preview-2024-12-17\",
     \"gpt-4o-audio-preview-2025-06-03\", \"gpt-4o-mini-audio-preview\",
     \"gpt-4o-mini-audio-preview-2024-12-17\", \"gpt-4o-search-preview\",
     \"gpt-4o-mini-search-preview\", \"gpt-4o-search-preview-2025-03-11\",
     \"gpt-4o-mini-search-preview-2025-03-11\", \"chatgpt-4o-latest\", \"codex-mini-latest\",
     \"gpt-4o-mini\", \"gpt-4o-mini-2024-07-18\", \"gpt-4-turbo\", \"gpt-4-turbo-2024-04-09\",
     \"gpt-4-0125-preview\", \"gpt-4-turbo-preview\", \"gpt-4-1106-preview\",
     \"gpt-4-vision-preview\", \"gpt-4\", \"gpt-4-0314\", \"gpt-4-0613\", \"gpt-4-32k\",
     \"gpt-4-32k-0314\", \"gpt-4-32k-0613\", \"gpt-3.5-turbo\", \"gpt-3.5-turbo-16k\",
     \"gpt-3.5-turbo-0301\", \"gpt-3.5-turbo-0613\", \"gpt-3.5-turbo-1106\", \"gpt-3.5-turbo-0125\",
     \"gpt-3.5-turbo-16k-0613\", \"o1-pro\", \"o1-pro-2025-03-19\", \"o3-pro\",
     \"o3-pro-2025-06-10\", \"o3-deep-research\", \"o3-deep-research-2025-06-26\",
     \"o4-mini-deep-research\", \"o4-mini-deep-research-2025-06-26\", \"computer-use-preview\", and
     \"computer-use-preview-2025-03-11\"."""
    instructions: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """A system (or developer) message inserted into the model's context."""
    temperature: Optional[float] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """What sampling temperature to use, between 0 and 2. Higher values like 0.8 will make the output
     more random, while lower values like 0.2 will make it more focused and deterministic.
     We generally recommend altering this or ``top_p`` but not both."""
    top_p: Optional[float] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """An alternative to sampling with temperature, called nucleus sampling,
     where the model considers the results of the tokens with top_p probability
     mass. So 0.1 means only the tokens comprising the top 10% probability mass
     are considered.
     
     We generally recommend altering this or ``temperature`` but not both."""
    reasoning: Optional["_openai_models3.Reasoning"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    tools: Optional[List["_openai_models3.Tool"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """An array of tools the model may call while generating a response. You
     can specify which tool to use by setting the ``tool_choice`` parameter."""
    text: Optional["_models.PromptAgentDefinitionText"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Configuration options for a text response from the model. Can be plain text or structured JSON
     data."""
    structured_inputs: Optional[Dict[str, "_models.StructuredInputDefinition"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Set of structured inputs that can participate in prompt template substitution or tool argument
     bindings."""

    @overload
    def __init__(
        self,
        *,
        model: Optional[Union[str, "_openai_models3.ModelIdsResponses"]] = None,
        instructions: Optional[str] = None,
        temperature: Optional[float] = None,
        top_p: Optional[float] = None,
        reasoning: Optional["_openai_models3.Reasoning"] = None,
        tools: Optional[List["_openai_models3.Tool"]] = None,
        text: Optional["_models.PromptAgentDefinitionText"] = None,
        structured_inputs: Optional[Dict[str, "_models.StructuredInputDefinition"]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=AgentKind.PROMPT_AGENT, **kwargs)


class PromptAgentDefinitionText(_Model):
    """PromptAgentDefinitionText.

    :ivar format:
    :vartype format: ~openai.models.ResponseTextFormatConfiguration
    """

    format: Optional["_openai_models3.ResponseTextFormatConfiguration"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )

    @overload
    def __init__(
        self,
        *,
        format: Optional["_openai_models3.ResponseTextFormatConfiguration"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class Response(_Model):
    """Response.

    :ivar agent: The agent used for this response. Required.
    :vartype agent: ~azureaiagents.models.AgentId
    :ivar conversation_id: The id of the conversation used for this response.
    :vartype conversation_id: str
    :ivar metadata: Set of 16 key-value pairs that can be attached to an object. This can be
     useful for storing additional information about the object in a structured
     format, and querying for objects via API or the dashboard.

     Keys are strings with a maximum length of 64 characters. Values are strings
     with a maximum length of 512 characters. Required.
    :vartype metadata: dict[str, str]
    :ivar temperature: What sampling temperature to use, between 0 and 2. Higher values like 0.8
     will make the output more random, while lower values like 0.2 will make it more focused and
     deterministic.
     We generally recommend altering this or ``top_p`` but not both. Required.
    :vartype temperature: float
    :ivar top_p: An alternative to sampling with temperature, called nucleus sampling,
     where the model considers the results of the tokens with top_p probability
     mass. So 0.1 means only the tokens comprising the top 10% probability mass
     are considered.

     We generally recommend altering this or ``temperature`` but not both. Required.
    :vartype top_p: float
    :ivar user: A unique identifier representing your end-user, which can help OpenAI to monitor
     and detect abuse. `Learn more </docs/guides/safety-best-practices#end-user-ids>`_. Required.
    :vartype user: str
    :ivar service_tier: Known values are: "auto", "default", "flex", "scale", and "priority".
    :vartype service_tier: str or ~openai.models.ServiceTier
    :ivar top_logprobs: An integer between 0 and 20 specifying the number of most likely tokens to
     return at each token position, each with an associated log probability.
    :vartype top_logprobs: int
    :ivar previous_response_id: The unique ID of the previous response to the model. Use this to
     create multi-turn conversations. Learn more about
     `conversation state </docs/guides/conversation-state>`_.
    :vartype previous_response_id: str
    :ivar model: Model ID used to generate the response, like ``gpt-4o`` or ``o3``. OpenAI
     offers a wide range of models with different capabilities, performance
     characteristics, and price points. Refer to the `model guide </docs/models>`_
     to browse and compare available models. Known values are: "gpt-4.1", "gpt-4.1-mini",
     "gpt-4.1-nano", "gpt-4.1-2025-04-14", "gpt-4.1-mini-2025-04-14", "gpt-4.1-nano-2025-04-14",
     "o4-mini", "o4-mini-2025-04-16", "o3", "o3-2025-04-16", "o3-mini", "o3-mini-2025-01-31", "o1",
     "o1-2024-12-17", "o1-preview", "o1-preview-2024-09-12", "o1-mini", "o1-mini-2024-09-12",
     "gpt-4o", "gpt-4o-2024-11-20", "gpt-4o-2024-08-06", "gpt-4o-2024-05-13",
     "gpt-4o-audio-preview", "gpt-4o-audio-preview-2024-10-01", "gpt-4o-audio-preview-2024-12-17",
     "gpt-4o-audio-preview-2025-06-03", "gpt-4o-mini-audio-preview",
     "gpt-4o-mini-audio-preview-2024-12-17", "gpt-4o-search-preview", "gpt-4o-mini-search-preview",
     "gpt-4o-search-preview-2025-03-11", "gpt-4o-mini-search-preview-2025-03-11",
     "chatgpt-4o-latest", "codex-mini-latest", "gpt-4o-mini", "gpt-4o-mini-2024-07-18",
     "gpt-4-turbo", "gpt-4-turbo-2024-04-09", "gpt-4-0125-preview", "gpt-4-turbo-preview",
     "gpt-4-1106-preview", "gpt-4-vision-preview", "gpt-4", "gpt-4-0314", "gpt-4-0613", "gpt-4-32k",
     "gpt-4-32k-0314", "gpt-4-32k-0613", "gpt-3.5-turbo", "gpt-3.5-turbo-16k", "gpt-3.5-turbo-0301",
     "gpt-3.5-turbo-0613", "gpt-3.5-turbo-1106", "gpt-3.5-turbo-0125", "gpt-3.5-turbo-16k-0613",
     "o1-pro", "o1-pro-2025-03-19", "o3-pro", "o3-pro-2025-06-10", "o3-deep-research",
     "o3-deep-research-2025-06-26", "o4-mini-deep-research", "o4-mini-deep-research-2025-06-26",
     "computer-use-preview", and "computer-use-preview-2025-03-11".
    :vartype model: str or ~openai.models.ModelIdsResponses
    :ivar reasoning:
    :vartype reasoning: ~openai.models.Reasoning
    :ivar background: Whether to run the model response in the background.
     `Learn more </docs/guides/background>`_.
    :vartype background: bool
    :ivar max_output_tokens: An upper bound for the number of tokens that can be generated for a
     response, including visible output tokens and `reasoning tokens </docs/guides/reasoning>`_.
    :vartype max_output_tokens: int
    :ivar max_tool_calls: The maximum number of total calls to built-in tools that can be processed
     in a response. This maximum number applies across all built-in tool calls, not per individual
     tool. Any further attempts to call a tool by the model will be ignored.
    :vartype max_tool_calls: int
    :ivar text: Configuration options for a text response from the model. Can be plain
     text or structured JSON data. Learn more:

     * [Text inputs and outputs](/docs/guides/text)
     * [Structured Outputs](/docs/guides/structured-outputs).
    :vartype text: ~openai.models.CreateResponseText
    :ivar tools: An array of tools the model may call while generating a response. You
     can specify which tool to use by setting the ``tool_choice`` parameter.

     The two categories of tools you can provide the model are:



     * **Built-in tools**: Tools that are provided by OpenAI that extend the
     model's capabilities, like [web search](/docs/guides/tools-web-search)
     or [file search](/docs/guides/tools-file-search). Learn more about
     [built-in tools](/docs/guides/tools).
     * **Function calls (custom tools)**: Functions that are defined by you,
     enabling the model to call your own code. Learn more about
     [function calling](/docs/guides/function-calling).
    :vartype tools: list[~openai.models.Tool]
    :ivar tool_choice: How the model should select which tool (or tools) to use when generating
     a response. See the ``tools`` parameter to see how to specify which tools
     the model can call. Is either a Union[str, "_openai_models2.ToolChoiceOptions"] type or a
     ToolChoiceObject type.
    :vartype tool_choice: str or ~openai.models.ToolChoiceOptions or
     ~openai.models.ToolChoiceObject
    :ivar prompt:
    :vartype prompt: ~openai.models.Prompt
    :ivar truncation: The truncation strategy to use for the model response.

     * `auto`: If the context of this response and previous ones exceeds
     the model's context window size, the model will truncate the
     response to fit the context window by dropping input items in the
     middle of the conversation.
     * `disabled` (default): If a model response will exceed the context window
     size for a model, the request will fail with a 400 error. Is either a Literal["auto"] type or a
     Literal["disabled"] type.
    :vartype truncation: str or str
    :ivar id: Unique identifier for this Response. Required.
    :vartype id: str
    :ivar object: The object type of this resource - always set to ``response``. Required. Default
     value is "response".
    :vartype object: str
    :ivar status: The status of the response generation. One of ``completed``, ``failed``,
     ``in_progress``, ``cancelled``, ``queued``, or ``incomplete``. Is one of the following types:
     Literal["completed"], Literal["failed"], Literal["in_progress"], Literal["cancelled"],
     Literal["queued"], Literal["incomplete"]
    :vartype status: str or str or str or str or str or str
    :ivar created_at: Unix timestamp (in seconds) of when this Response was created. Required.
    :vartype created_at: ~datetime.datetime
    :ivar error: Required.
    :vartype error: ~openai.models.ResponseError
    :ivar incomplete_details: Details about why the response is incomplete. Required.
    :vartype incomplete_details: ~openai.models.ResponseIncompleteDetails1
    :ivar output: An array of content items generated by the model.



     * The length and order of items in the `output` array is dependent
     on the model's response.
     * Rather than accessing the first item in the `output` array and
     assuming it's an `assistant` message with the content generated by
     the model, you might consider using the `output_text` property where
     supported in SDKs. Required.
    :vartype output: list[~openai.models.ItemResource]
    :ivar instructions: A system (or developer) message inserted into the model's context.

     When using along with ``previous_response_id``, the instructions from a previous
     response will not be carried over to the next response. This makes it simple
     to swap out system (or developer) messages in new responses. Required. Is either a str type or
     a [ItemParam] type.
    :vartype instructions: str or list[~openai.models.ItemParam]
    :ivar output_text: SDK-only convenience property that contains the aggregated text output
     from all ``output_text`` items in the ``output`` array, if any are present.
     Supported in the Python and JavaScript SDKs.
    :vartype output_text: str
    :ivar usage:
    :vartype usage: ~openai.models.ResponseUsage
    :ivar parallel_tool_calls: Whether to allow the model to run tool calls in parallel. Required.
    :vartype parallel_tool_calls: bool
    """

    agent: "_models.AgentId" = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The agent used for this response. Required."""
    conversation_id: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The id of the conversation used for this response."""
    metadata: Dict[str, str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Set of 16 key-value pairs that can be attached to an object. This can be
     useful for storing additional information about the object in a structured
     format, and querying for objects via API or the dashboard.
     
     Keys are strings with a maximum length of 64 characters. Values are strings
     with a maximum length of 512 characters. Required."""
    temperature: float = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """What sampling temperature to use, between 0 and 2. Higher values like 0.8 will make the output
     more random, while lower values like 0.2 will make it more focused and deterministic.
     We generally recommend altering this or ``top_p`` but not both. Required."""
    top_p: float = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """An alternative to sampling with temperature, called nucleus sampling,
     where the model considers the results of the tokens with top_p probability
     mass. So 0.1 means only the tokens comprising the top 10% probability mass
     are considered.
     
     We generally recommend altering this or ``temperature`` but not both. Required."""
    user: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """A unique identifier representing your end-user, which can help OpenAI to monitor and detect
     abuse. `Learn more </docs/guides/safety-best-practices#end-user-ids>`_. Required."""
    service_tier: Optional[Union[str, "_openai_models3.ServiceTier"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Known values are: \"auto\", \"default\", \"flex\", \"scale\", and \"priority\"."""
    top_logprobs: Optional[int] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """An integer between 0 and 20 specifying the number of most likely tokens to return at each token
     position, each with an associated log probability."""
    previous_response_id: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The unique ID of the previous response to the model. Use this to
     create multi-turn conversations. Learn more about
     `conversation state </docs/guides/conversation-state>`_."""
    model: Optional[Union[str, "_openai_models3.ModelIdsResponses"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Model ID used to generate the response, like ``gpt-4o`` or ``o3``. OpenAI
     offers a wide range of models with different capabilities, performance
     characteristics, and price points. Refer to the `model guide </docs/models>`_
     to browse and compare available models. Known values are: \"gpt-4.1\", \"gpt-4.1-mini\",
     \"gpt-4.1-nano\", \"gpt-4.1-2025-04-14\", \"gpt-4.1-mini-2025-04-14\",
     \"gpt-4.1-nano-2025-04-14\", \"o4-mini\", \"o4-mini-2025-04-16\", \"o3\", \"o3-2025-04-16\",
     \"o3-mini\", \"o3-mini-2025-01-31\", \"o1\", \"o1-2024-12-17\", \"o1-preview\",
     \"o1-preview-2024-09-12\", \"o1-mini\", \"o1-mini-2024-09-12\", \"gpt-4o\",
     \"gpt-4o-2024-11-20\", \"gpt-4o-2024-08-06\", \"gpt-4o-2024-05-13\", \"gpt-4o-audio-preview\",
     \"gpt-4o-audio-preview-2024-10-01\", \"gpt-4o-audio-preview-2024-12-17\",
     \"gpt-4o-audio-preview-2025-06-03\", \"gpt-4o-mini-audio-preview\",
     \"gpt-4o-mini-audio-preview-2024-12-17\", \"gpt-4o-search-preview\",
     \"gpt-4o-mini-search-preview\", \"gpt-4o-search-preview-2025-03-11\",
     \"gpt-4o-mini-search-preview-2025-03-11\", \"chatgpt-4o-latest\", \"codex-mini-latest\",
     \"gpt-4o-mini\", \"gpt-4o-mini-2024-07-18\", \"gpt-4-turbo\", \"gpt-4-turbo-2024-04-09\",
     \"gpt-4-0125-preview\", \"gpt-4-turbo-preview\", \"gpt-4-1106-preview\",
     \"gpt-4-vision-preview\", \"gpt-4\", \"gpt-4-0314\", \"gpt-4-0613\", \"gpt-4-32k\",
     \"gpt-4-32k-0314\", \"gpt-4-32k-0613\", \"gpt-3.5-turbo\", \"gpt-3.5-turbo-16k\",
     \"gpt-3.5-turbo-0301\", \"gpt-3.5-turbo-0613\", \"gpt-3.5-turbo-1106\", \"gpt-3.5-turbo-0125\",
     \"gpt-3.5-turbo-16k-0613\", \"o1-pro\", \"o1-pro-2025-03-19\", \"o3-pro\",
     \"o3-pro-2025-06-10\", \"o3-deep-research\", \"o3-deep-research-2025-06-26\",
     \"o4-mini-deep-research\", \"o4-mini-deep-research-2025-06-26\", \"computer-use-preview\", and
     \"computer-use-preview-2025-03-11\"."""
    reasoning: Optional["_openai_models3.Reasoning"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    background: Optional[bool] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Whether to run the model response in the background.
     `Learn more </docs/guides/background>`_."""
    max_output_tokens: Optional[int] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """An upper bound for the number of tokens that can be generated for a response, including visible
     output tokens and `reasoning tokens </docs/guides/reasoning>`_."""
    max_tool_calls: Optional[int] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The maximum number of total calls to built-in tools that can be processed in a response. This
     maximum number applies across all built-in tool calls, not per individual tool. Any further
     attempts to call a tool by the model will be ignored."""
    text: Optional["_openai_models3.CreateResponseText"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Configuration options for a text response from the model. Can be plain
     text or structured JSON data. Learn more:
     
     * [Text inputs and outputs](/docs/guides/text)
     * [Structured Outputs](/docs/guides/structured-outputs)."""
    tools: Optional[List["_openai_models3.Tool"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """An array of tools the model may call while generating a response. You
     can specify which tool to use by setting the ``tool_choice`` parameter.
     
     The two categories of tools you can provide the model are:
     
     
     
     * **Built-in tools**: Tools that are provided by OpenAI that extend the
     model's capabilities, like [web search](/docs/guides/tools-web-search)
     or [file search](/docs/guides/tools-file-search). Learn more about
     [built-in tools](/docs/guides/tools).
     * **Function calls (custom tools)**: Functions that are defined by you,
     enabling the model to call your own code. Learn more about
     [function calling](/docs/guides/function-calling)."""
    tool_choice: Optional[Union[str, "_openai_models3.ToolChoiceOptions", "_openai_models3.ToolChoiceObject"]] = (
        rest_field(visibility=["read", "create", "update", "delete", "query"])
    )
    """How the model should select which tool (or tools) to use when generating
     a response. See the ``tools`` parameter to see how to specify which tools
     the model can call. Is either a Union[str, \"_openai_models2.ToolChoiceOptions\"] type or a
     ToolChoiceObject type."""
    prompt: Optional["_openai_models3.Prompt"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    truncation: Optional[Literal["auto", "disabled"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """The truncation strategy to use for the model response.
     
     * `auto`: If the context of this response and previous ones exceeds
     the model's context window size, the model will truncate the
     response to fit the context window by dropping input items in the
     middle of the conversation.
     * `disabled` (default): If a model response will exceed the context window
     size for a model, the request will fail with a 400 error. Is either a Literal[\"auto\"] type or
     a Literal[\"disabled\"] type."""
    id: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Unique identifier for this Response. Required."""
    object: Literal["response"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The object type of this resource - always set to ``response``. Required. Default value is
     \"response\"."""
    status: Optional[Literal["completed", "failed", "in_progress", "cancelled", "queued", "incomplete"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """The status of the response generation. One of ``completed``, ``failed``,
     ``in_progress``, ``cancelled``, ``queued``, or ``incomplete``. Is one of the following types:
     Literal[\"completed\"], Literal[\"failed\"], Literal[\"in_progress\"], Literal[\"cancelled\"],
     Literal[\"queued\"], Literal[\"incomplete\"]"""
    created_at: datetime.datetime = rest_field(
        visibility=["read", "create", "update", "delete", "query"], format="unix-timestamp"
    )
    """Unix timestamp (in seconds) of when this Response was created. Required."""
    error: "_openai_models3.ResponseError" = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Required."""
    incomplete_details: "_openai_models3.ResponseIncompleteDetails1" = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Details about why the response is incomplete. Required."""
    output: List["_openai_models3.ItemResource"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """An array of content items generated by the model.
     
     
     
     * The length and order of items in the `output` array is dependent
     on the model's response.
     * Rather than accessing the first item in the `output` array and
     assuming it's an `assistant` message with the content generated by
     the model, you might consider using the `output_text` property where
     supported in SDKs. Required."""
    instructions: Union[str, List["_openai_models3.ItemParam"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """A system (or developer) message inserted into the model's context.
     
     When using along with ``previous_response_id``, the instructions from a previous
     response will not be carried over to the next response. This makes it simple
     to swap out system (or developer) messages in new responses. Required. Is either a str type or
     a [ItemParam] type."""
    output_text: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """SDK-only convenience property that contains the aggregated text output
     from all ``output_text`` items in the ``output`` array, if any are present.
     Supported in the Python and JavaScript SDKs."""
    usage: Optional["_openai_models3.ResponseUsage"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    parallel_tool_calls: bool = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Whether to allow the model to run tool calls in parallel. Required."""

    @overload
    def __init__(  # pylint: disable=too-many-locals
        self,
        *,
        agent: "_models.AgentId",
        metadata: Dict[str, str],
        temperature: float,
        top_p: float,
        user: str,
        id: str,  # pylint: disable=redefined-builtin
        created_at: datetime.datetime,
        error: "_openai_models3.ResponseError",
        incomplete_details: "_openai_models3.ResponseIncompleteDetails1",
        output: List["_openai_models3.ItemResource"],
        instructions: Union[str, List["_openai_models3.ItemParam"]],
        parallel_tool_calls: bool,
        conversation_id: Optional[str] = None,
        service_tier: Optional[Union[str, "_openai_models3.ServiceTier"]] = None,
        top_logprobs: Optional[int] = None,
        previous_response_id: Optional[str] = None,
        model: Optional[Union[str, "_openai_models3.ModelIdsResponses"]] = None,
        reasoning: Optional["_openai_models3.Reasoning"] = None,
        background: Optional[bool] = None,
        max_output_tokens: Optional[int] = None,
        max_tool_calls: Optional[int] = None,
        text: Optional["_openai_models3.CreateResponseText"] = None,
        tools: Optional[List["_openai_models3.Tool"]] = None,
        tool_choice: Optional[
            Union[str, "_openai_models3.ToolChoiceOptions", "_openai_models3.ToolChoiceObject"]
        ] = None,
        prompt: Optional["_openai_models3.Prompt"] = None,
        truncation: Optional[Literal["auto", "disabled"]] = None,
        status: Optional[Literal["completed", "failed", "in_progress", "cancelled", "queued", "incomplete"]] = None,
        output_text: Optional[str] = None,
        usage: Optional["_openai_models3.ResponseUsage"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.object: Literal["response"] = "response"


class ResponseList(_Model):
    """ResponseList.

    :ivar object: The type of object returned, must be ``list``. Required. Default value is "list".
    :vartype object: str
    :ivar data: A list of items used to generate this response. Required.
    :vartype data: list[~openai.models.Response]
    :ivar has_more: Whether there are more items available. Required.
    :vartype has_more: bool
    :ivar first_id: The ID of the first item in the list. Required.
    :vartype first_id: str
    :ivar last_id: The ID of the last item in the list. Required.
    :vartype last_id: str
    """

    object: Literal["list"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The type of object returned, must be ``list``. Required. Default value is \"list\"."""
    data: List["_openai_models3.Response"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """A list of items used to generate this response. Required."""
    has_more: bool = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Whether there are more items available. Required."""
    first_id: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The ID of the first item in the list. Required."""
    last_id: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The ID of the last item in the list. Required."""

    @overload
    def __init__(
        self,
        *,
        data: List["_openai_models3.Response"],
        has_more: bool,
        first_id: str,
        last_id: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.object: Literal["list"] = "list"


class SemanticEventDefinition(_Model):
    """An event that can be raised by the agent based on a semantic condition.

    :ivar condition: A condition that specifies when the event must be raised. Used by the model to
     determine when to raise the event. Required.
    :vartype condition: str
    """

    condition: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """A condition that specifies when the event must be raised. Used by the model to determine when
     to raise the event. Required."""

    @overload
    def __init__(
        self,
        *,
        condition: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class SemanticEventsOutputItemResource(ItemResource, discriminator="semantic_event"):
    """SemanticEventsOutputItemResource.

    :ivar id: Required.
    :vartype id: str
    :ivar type: Required. Default value is "semantic_event".
    :vartype type: str
    :ivar name: The name of the semantic event. Required.
    :vartype name: str
    """

    type: Literal["semantic_event"] = rest_discriminator(name="type", visibility=["read", "create", "update", "delete", "query"])  # type: ignore
    """Required. Default value is \"semantic_event\"."""
    name: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The name of the semantic event. Required."""

    @overload
    def __init__(
        self,
        *,
        id: str,  # pylint: disable=redefined-builtin
        name: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, type="semantic_event", **kwargs)


class StructuredInputDefinition(_Model):
    """An structured input that can participate in prompt template substitutions and tool argument
    binding.

    :ivar description: A human-readable description of the input.
    :vartype description: str
    :ivar default_value: The default value for the input if no run-time value is provided.
    :vartype default_value: any
    :ivar tool_argument_bindings: When provided, the input value is binded to the specified tool
     arguments.
    :vartype tool_argument_bindings: list[~azureaiagents.models.ToolArgumentBinding]
    :ivar schema: The JSON schema for the structured input (optional).
    :vartype schema: any
    :ivar required: Whether the input property is required when the agent is invoked.
    :vartype required: bool
    """

    description: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """A human-readable description of the input."""
    default_value: Optional[Any] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The default value for the input if no run-time value is provided."""
    tool_argument_bindings: Optional[List["_models.ToolArgumentBinding"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """When provided, the input value is binded to the specified tool arguments."""
    schema: Optional[Any] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The JSON schema for the structured input (optional)."""
    required: Optional[bool] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Whether the input property is required when the agent is invoked."""

    @overload
    def __init__(
        self,
        *,
        description: Optional[str] = None,
        default_value: Optional[Any] = None,
        tool_argument_bindings: Optional[List["_models.ToolArgumentBinding"]] = None,
        schema: Optional[Any] = None,
        required: Optional[bool] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class StructuredInputsItemParam(ItemParam, discriminator="structured_inputs"):
    """StructuredInputsItemParam.

    :ivar type: Required. Default value is "structured_inputs".
    :vartype type: str
    :ivar inputs: The structured inputs to the response.
    :vartype inputs: dict[str, any]
    """

    type: Literal["structured_inputs"] = rest_discriminator(name="type", visibility=["read", "create", "update", "delete", "query"])  # type: ignore
    """Required. Default value is \"structured_inputs\"."""
    inputs: Optional[Dict[str, Any]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The structured inputs to the response."""

    @overload
    def __init__(
        self,
        *,
        inputs: Optional[Dict[str, Any]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, type="structured_inputs", **kwargs)


class StructuredInputsItemResource(ItemResource, discriminator="structured_inputs"):
    """StructuredInputsItemResource.

    :ivar id: Required.
    :vartype id: str
    :ivar type: Required. Default value is "structured_inputs".
    :vartype type: str
    :ivar inputs: The structured inputs provided to the response.
    :vartype inputs: dict[str, any]
    """

    type: Literal["structured_inputs"] = rest_discriminator(name="type", visibility=["read", "create", "update", "delete", "query"])  # type: ignore
    """Required. Default value is \"structured_inputs\"."""
    inputs: Optional[Dict[str, Any]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The structured inputs provided to the response."""

    @overload
    def __init__(
        self,
        *,
        id: str,  # pylint: disable=redefined-builtin
        inputs: Optional[Dict[str, Any]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, type="structured_inputs", **kwargs)


class StructuredOutputDefinition(_Model):
    """A structeud output that can be produced by the agent.

    :ivar description: A descriptopn of the output to emit. Used by the model to determine when to
     emit the output.
    :vartype description: str
    :ivar schema: The JSON schema for the structured output. Required.
    :vartype schema: dict[str, any]
    """

    description: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """A descriptopn of the output to emit. Used by the model to determine when to emit the output."""
    schema: Dict[str, Any] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The JSON schema for the structured output. Required."""

    @overload
    def __init__(
        self,
        *,
        schema: Dict[str, Any],
        description: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class StructuredOutputsItemResource(ItemResource, discriminator="structured_outputs"):
    """StructuredOutputsItemResource.

    :ivar id: Required.
    :vartype id: str
    :ivar type: Required. Default value is "structured_outputs".
    :vartype type: str
    :ivar outputs: The structured outputs captured during the response.
    :vartype outputs: dict[str, any]
    """

    type: Literal["structured_outputs"] = rest_discriminator(name="type", visibility=["read", "create", "update", "delete", "query"])  # type: ignore
    """Required. Default value is \"structured_outputs\"."""
    outputs: Optional[Dict[str, Any]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The structured outputs captured during the response."""

    @overload
    def __init__(
        self,
        *,
        id: str,  # pylint: disable=redefined-builtin
        outputs: Optional[Dict[str, Any]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, type="structured_outputs", **kwargs)


class ToolArgumentBinding(_Model):
    """ToolArgumentBinding.

    :ivar tool_name: The name of the tool to participate in the argument binding. If not provided,
     then all tools with matching arguments will participate in binding.
    :vartype tool_name: str
    :ivar argument_name: The name of the argument within the tool. Required.
    :vartype argument_name: str
    """

    tool_name: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The name of the tool to participate in the argument binding. If not provided, then all tools
     with matching arguments will participate in binding."""
    argument_name: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The name of the argument within the tool. Required."""

    @overload
    def __init__(
        self,
        *,
        argument_name: str,
        tool_name: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class UpsertAgentLabelRequest(_Model):
    """UpsertAgentLabelRequest.

    :ivar associated_version: The version of the agent that this label is associated with. If not
     provided, then the label is not associated with any specific version.
    :vartype associated_version: str
    """

    associated_version: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The version of the agent that this label is associated with. If not provided, then the label is
     not associated with any specific version."""

    @overload
    def __init__(
        self,
        *,
        associated_version: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class WorkflowDefinition(AgentDefinition, discriminator="workflow"):
    """The workflow specification in CPSDL format.

    :ivar kind: Required.
    :vartype kind: str or ~azureaiagents.models.WORKFLOW
    :ivar actions: The workflow actions in CPSDL format. Required.
    :vartype actions: list[any]
    """

    kind: Literal[AgentKind.WORKFLOW] = rest_discriminator(name="kind", visibility=["read", "create", "update", "delete", "query"])  # type: ignore
    """Required."""
    actions: List[Any] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The workflow actions in CPSDL format. Required."""

    @overload
    def __init__(
        self,
        *,
        actions: List[Any],
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=AgentKind.WORKFLOW, **kwargs)



from .langgraph_request_converter import LangGraphRequestConverter
from .langgraph_response_converter import LangGraphResponseConverter
from .langgraph_stream_async_response_converter import LangGraphStreamAsyncResponseConverter
from .langgraph_stream_response_converter import LangGraphStreamResponseConverter

__all__ = [
    "LangGraphRequestConverter",
    "LangGraphResponseConverter",
    "LangGraphStreamAsyncResponseConverter",
    "LangGraphStreamResponseConverter",
]



class Constants:
    # well-known environment variables
    APPLICATION_INSIGHTS_CONNECTION_STRING = "_AGENT_RUNTIME_APP_INSIGHTS_CONNECTION_STRING"
    AGENT_ID = "AGENT_ID"
    AGENT_PROJECT_RESOURCE_ID = "AGENT_PROJECT_NAME"



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class AnnotationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of AnnotationType."""

    FILE_CITATION = "file_citation"
    URL_CITATION = "url_citation"
    FILE_PATH = "file_path"
    CONTAINER_FILE_CITATION = "container_file_citation"


class CodeInterpreterOutputType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of CodeInterpreterOutputType."""

    LOGS = "logs"
    IMAGE = "image"


class ComputerActionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of ComputerActionType."""

    SCREENSHOT = "screenshot"
    CLICK = "click"
    DOUBLE_CLICK = "double_click"
    SCROLL = "scroll"
    TYPE = "type"
    WAIT = "wait"
    KEYPRESS = "keypress"
    DRAG = "drag"
    MOVE = "move"


class ComputerToolCallOutputItemOutputType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """A computer screenshot image used with the computer use tool."""

    SCREENSHOT = "computer_screenshot"


class Includable(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specify additional output data to include in the model response. Currently
    supported values are:

    * `code_interpreter_call.outputs`: Includes the outputs of python code execution
    in code interpreter tool call items.
    * `computer_call_output.output.image_url`: Include image urls from the computer call output.
    * `file_search_call.results`: Include the search results of
    the file search tool call.
    * `message.input_image.image_url`: Include image urls from the input message.
    * `message.output_text.logprobs`: Include logprobs with assistant messages.
    * `reasoning.encrypted_content`: Includes an encrypted version of reasoning
    tokens in reasoning item outputs. This enables reasoning items to be used in
    multi-turn conversations when using the Responses API statelessly (like
    when the `store` parameter is set to `false`, or when an organization is
    enrolled in the zero data retention program).
    """

    CODE_INTERPRETER_CALL_OUTPUTS = "code_interpreter_call.outputs"
    COMPUTER_CALL_OUTPUT_OUTPUT_IMAGE_URL = "computer_call_output.output.image_url"
    FILE_SEARCH_CALL_RESULTS = "file_search_call.results"
    MESSAGE_INPUT_IMAGE_IMAGE_URL = "message.input_image.image_url"
    MESSAGE_OUTPUT_TEXT_LOGPROBS = "message.output_text.logprobs"
    REASONING_ENCRYPTED_CONTENT = "reasoning.encrypted_content"


class ItemContentType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Multi-modal input and output contents."""

    INPUT_TEXT = "input_text"
    INPUT_AUDIO = "input_audio"
    INPUT_IMAGE = "input_image"
    INPUT_FILE = "input_file"
    OUTPUT_TEXT = "output_text"
    OUTPUT_AUDIO = "output_audio"
    REFUSAL = "refusal"


class ItemType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of ItemType."""

    MESSAGE = "message"
    FILE_SEARCH_CALL = "file_search_call"
    FUNCTION_CALL = "function_call"
    FUNCTION_CALL_OUTPUT = "function_call_output"
    COMPUTER_CALL = "computer_call"
    COMPUTER_CALL_OUTPUT = "computer_call_output"
    WEB_SEARCH_CALL = "web_search_call"
    REASONING = "reasoning"
    ITEM_REFERENCE = "item_reference"
    IMAGE_GENERATION_CALL = "image_generation_call"
    CODE_INTERPRETER_CALL = "code_interpreter_call"
    LOCAL_SHELL_CALL = "local_shell_call"
    LOCAL_SHELL_CALL_OUTPUT = "local_shell_call_output"
    MCP_LIST_TOOLS = "mcp_list_tools"
    MCP_APPROVAL_REQUEST = "mcp_approval_request"
    MCP_APPROVAL_RESPONSE = "mcp_approval_response"
    MCP_CALL = "mcp_call"


class LocationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of LocationType."""

    APPROXIMATE = "approximate"


class ModelIdsResponses(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of ModelIdsResponses."""

    GPT4_1 = "gpt-4.1"
    GPT4_1_MINI = "gpt-4.1-mini"
    GPT4_1_NANO = "gpt-4.1-nano"
    GPT4_1_2025_04_14 = "gpt-4.1-2025-04-14"
    GPT4_1_MINI2025_04_14 = "gpt-4.1-mini-2025-04-14"
    GPT4_1_NANO2025_04_14 = "gpt-4.1-nano-2025-04-14"
    O4_MINI = "o4-mini"
    O4_MINI2025_04_16 = "o4-mini-2025-04-16"
    O3 = "o3"
    O3_2025_04_16 = "o3-2025-04-16"
    O3_MINI = "o3-mini"
    O3_MINI2025_01_31 = "o3-mini-2025-01-31"
    O1 = "o1"
    O1_2024_12_17 = "o1-2024-12-17"
    O1_PREVIEW = "o1-preview"
    O1_PREVIEW2024_09_12 = "o1-preview-2024-09-12"
    O1_MINI = "o1-mini"
    O1_MINI2024_09_12 = "o1-mini-2024-09-12"
    GPT4_O = "gpt-4o"
    GPT4_O2024_11_20 = "gpt-4o-2024-11-20"
    GPT4_O2024_08_06 = "gpt-4o-2024-08-06"
    GPT4_O2024_05_13 = "gpt-4o-2024-05-13"
    GPT4_O_AUDIO_PREVIEW = "gpt-4o-audio-preview"
    GPT4_O_AUDIO_PREVIEW2024_10_01 = "gpt-4o-audio-preview-2024-10-01"
    GPT4_O_AUDIO_PREVIEW2024_12_17 = "gpt-4o-audio-preview-2024-12-17"
    GPT4_O_AUDIO_PREVIEW2025_06_03 = "gpt-4o-audio-preview-2025-06-03"
    GPT4_O_MINI_AUDIO_PREVIEW = "gpt-4o-mini-audio-preview"
    GPT4_O_MINI_AUDIO_PREVIEW2024_12_17 = "gpt-4o-mini-audio-preview-2024-12-17"
    GPT4_O_SEARCH_PREVIEW = "gpt-4o-search-preview"
    GPT4_O_MINI_SEARCH_PREVIEW = "gpt-4o-mini-search-preview"
    GPT4_O_SEARCH_PREVIEW2025_03_11 = "gpt-4o-search-preview-2025-03-11"
    GPT4_O_MINI_SEARCH_PREVIEW2025_03_11 = "gpt-4o-mini-search-preview-2025-03-11"
    CHATGPT4_O_LATEST = "chatgpt-4o-latest"
    CODEX_MINI_LATEST = "codex-mini-latest"
    GPT4_O_MINI = "gpt-4o-mini"
    GPT4_O_MINI2024_07_18 = "gpt-4o-mini-2024-07-18"
    GPT4_TURBO = "gpt-4-turbo"
    GPT4_TURBO2024_04_09 = "gpt-4-turbo-2024-04-09"
    GPT4_0125_PREVIEW = "gpt-4-0125-preview"
    GPT4_TURBO_PREVIEW = "gpt-4-turbo-preview"
    GPT4_1106_PREVIEW = "gpt-4-1106-preview"
    GPT4_VISION_PREVIEW = "gpt-4-vision-preview"
    GPT4 = "gpt-4"
    GPT4_0314 = "gpt-4-0314"
    GPT4_0613 = "gpt-4-0613"
    GPT4_32_K = "gpt-4-32k"
    GPT4_32_K0314 = "gpt-4-32k-0314"
    GPT4_32_K0613 = "gpt-4-32k-0613"
    GPT3_5_TURBO = "gpt-3.5-turbo"
    GPT3_5_TURBO16_K = "gpt-3.5-turbo-16k"
    GPT3_5_TURBO0301 = "gpt-3.5-turbo-0301"
    GPT3_5_TURBO0613 = "gpt-3.5-turbo-0613"
    GPT3_5_TURBO1106 = "gpt-3.5-turbo-1106"
    GPT3_5_TURBO0125 = "gpt-3.5-turbo-0125"
    GPT3_5_TURBO16_K0613 = "gpt-3.5-turbo-16k-0613"
    O1_PRO = "o1-pro"
    O1_PRO2025_03_19 = "o1-pro-2025-03-19"
    O3_PRO = "o3-pro"
    O3_PRO2025_06_10 = "o3-pro-2025-06-10"
    O3_DEEP_RESEARCH = "o3-deep-research"
    O3_DEEP_RESEARCH2025_06_26 = "o3-deep-research-2025-06-26"
    O4_MINI_DEEP_RESEARCH = "o4-mini-deep-research"
    O4_MINI_DEEP_RESEARCH2025_06_26 = "o4-mini-deep-research-2025-06-26"
    COMPUTER_USE_PREVIEW = "computer-use-preview"
    COMPUTER_USE_PREVIEW2025_03_11 = "computer-use-preview-2025-03-11"


class ReasoningEffort(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """**o-series models only**

    Constrains effort on reasoning for
    `reasoning models <https://platform.openai.com/docs/guides/reasoning>`_.
    Currently supported values are ``low``, ``medium``, and ``high``. Reducing
    reasoning effort can result in faster responses and fewer tokens used
    on reasoning in a response.
    """

    LOW = "low"
    MEDIUM = "medium"
    HIGH = "high"


class ReasoningItemSummaryPartType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of ReasoningItemSummaryPartType."""

    SUMMARY_TEXT = "summary_text"


class ResponseErrorCode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The error code for the response."""

    SERVER_ERROR = "server_error"
    RATE_LIMIT_EXCEEDED = "rate_limit_exceeded"
    INVALID_PROMPT = "invalid_prompt"
    VECTOR_STORE_TIMEOUT = "vector_store_timeout"
    INVALID_IMAGE = "invalid_image"
    INVALID_IMAGE_FORMAT = "invalid_image_format"
    INVALID_BASE64_IMAGE = "invalid_base64_image"
    INVALID_IMAGE_URL = "invalid_image_url"
    IMAGE_TOO_LARGE = "image_too_large"
    IMAGE_TOO_SMALL = "image_too_small"
    IMAGE_PARSE_ERROR = "image_parse_error"
    IMAGE_CONTENT_POLICY_VIOLATION = "image_content_policy_violation"
    INVALID_IMAGE_MODE = "invalid_image_mode"
    IMAGE_FILE_TOO_LARGE = "image_file_too_large"
    UNSUPPORTED_IMAGE_MEDIA_TYPE = "unsupported_image_media_type"
    EMPTY_IMAGE_FILE = "empty_image_file"
    FAILED_TO_DOWNLOAD_IMAGE = "failed_to_download_image"
    IMAGE_FILE_NOT_FOUND = "image_file_not_found"


class ResponsesMessageRole(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The collection of valid roles for responses message items."""

    SYSTEM = "system"
    DEVELOPER = "developer"
    USER = "user"
    ASSISTANT = "assistant"


class ResponseStreamEventType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of ResponseStreamEventType."""

    RESPONSE_AUDIO_DELTA = "response.audio.delta"
    RESPONSE_AUDIO_DONE = "response.audio.done"
    RESPONSE_AUDIO_TRANSCRIPT_DELTA = "response.audio_transcript.delta"
    RESPONSE_AUDIO_TRANSCRIPT_DONE = "response.audio_transcript.done"
    RESPONSE_CODE_INTERPRETER_CALL_CODE_DELTA = "response.code_interpreter_call_code.delta"
    RESPONSE_CODE_INTERPRETER_CALL_CODE_DONE = "response.code_interpreter_call_code.done"
    RESPONSE_CODE_INTERPRETER_CALL_COMPLETED = "response.code_interpreter_call.completed"
    RESPONSE_CODE_INTERPRETER_CALL_IN_PROGRESS = "response.code_interpreter_call.in_progress"
    RESPONSE_CODE_INTERPRETER_CALL_INTERPRETING = "response.code_interpreter_call.interpreting"
    RESPONSE_COMPLETED = "response.completed"
    RESPONSE_CONTENT_PART_ADDED = "response.content_part.added"
    RESPONSE_CONTENT_PART_DONE = "response.content_part.done"
    RESPONSE_CREATED = "response.created"
    ERROR = "error"
    RESPONSE_FILE_SEARCH_CALL_COMPLETED = "response.file_search_call.completed"
    RESPONSE_FILE_SEARCH_CALL_IN_PROGRESS = "response.file_search_call.in_progress"
    RESPONSE_FILE_SEARCH_CALL_SEARCHING = "response.file_search_call.searching"
    RESPONSE_FUNCTION_CALL_ARGUMENTS_DELTA = "response.function_call_arguments.delta"
    RESPONSE_FUNCTION_CALL_ARGUMENTS_DONE = "response.function_call_arguments.done"
    RESPONSE_IN_PROGRESS = "response.in_progress"
    RESPONSE_FAILED = "response.failed"
    RESPONSE_INCOMPLETE = "response.incomplete"
    RESPONSE_OUTPUT_ITEM_ADDED = "response.output_item.added"
    RESPONSE_OUTPUT_ITEM_DONE = "response.output_item.done"
    RESPONSE_REFUSAL_DELTA = "response.refusal.delta"
    RESPONSE_REFUSAL_DONE = "response.refusal.done"
    RESPONSE_OUTPUT_TEXT_ANNOTATION_ADDED = "response.output_text.annotation.added"
    RESPONSE_OUTPUT_TEXT_DELTA = "response.output_text.delta"
    RESPONSE_OUTPUT_TEXT_DONE = "response.output_text.done"
    RESPONSE_REASONING_SUMMARY_PART_ADDED = "response.reasoning_summary_part.added"
    RESPONSE_REASONING_SUMMARY_PART_DONE = "response.reasoning_summary_part.done"
    RESPONSE_REASONING_SUMMARY_TEXT_DELTA = "response.reasoning_summary_text.delta"
    RESPONSE_REASONING_SUMMARY_TEXT_DONE = "response.reasoning_summary_text.done"
    RESPONSE_WEB_SEARCH_CALL_COMPLETED = "response.web_search_call.completed"
    RESPONSE_WEB_SEARCH_CALL_IN_PROGRESS = "response.web_search_call.in_progress"
    RESPONSE_WEB_SEARCH_CALL_SEARCHING = "response.web_search_call.searching"
    RESPONSE_IMAGE_GENERATION_CALL_COMPLETED = "response.image_generation_call.completed"
    RESPONSE_IMAGE_GENERATION_CALL_GENERATING = "response.image_generation_call.generating"
    RESPONSE_IMAGE_GENERATION_CALL_IN_PROGRESS = "response.image_generation_call.in_progress"
    RESPONSE_IMAGE_GENERATION_CALL_PARTIAL_IMAGE = "response.image_generation_call.partial_image"
    RESPONSE_MCP_CALL_ARGUMENTS_DELTA = "response.mcp_call.arguments_delta"
    RESPONSE_MCP_CALL_ARGUMENTS_DONE = "response.mcp_call.arguments_done"
    RESPONSE_MCP_CALL_COMPLETED = "response.mcp_call.completed"
    RESPONSE_MCP_CALL_FAILED = "response.mcp_call.failed"
    RESPONSE_MCP_CALL_IN_PROGRESS = "response.mcp_call.in_progress"
    RESPONSE_MCP_LIST_TOOLS_COMPLETED = "response.mcp_list_tools.completed"
    RESPONSE_MCP_LIST_TOOLS_FAILED = "response.mcp_list_tools.failed"
    RESPONSE_MCP_LIST_TOOLS_IN_PROGRESS = "response.mcp_list_tools.in_progress"
    RESPONSE_QUEUED = "response.queued"
    RESPONSE_REASONING_DELTA = "response.reasoning.delta"
    RESPONSE_REASONING_DONE = "response.reasoning.done"
    RESPONSE_REASONING_SUMMARY_DELTA = "response.reasoning_summary.delta"
    RESPONSE_REASONING_SUMMARY_DONE = "response.reasoning_summary.done"


class ResponseTextFormatConfigurationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """An object specifying the format that the model must output.

    Configuring ``{ "type": "json_schema" }`` enables Structured Outputs,
    which ensures the model will match your supplied JSON schema. Learn more in the
    `Structured Outputs guide </docs/guides/structured-outputs>`_.

    The default format is ``{ "type": "text" }`` with no additional options.

    **Not recommended for gpt-4o and newer models:**

    Setting to ``{ "type": "json_object" }`` enables the older JSON mode, which
    ensures the message the model generates is valid JSON. Using ``json_schema``
    is preferred for models that support it.
    """

    TEXT = "text"
    JSON_SCHEMA = "json_schema"
    JSON_OBJECT = "json_object"


class ServiceTier(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the processing type used for serving the request.

    * If set to 'auto', then the request will be processed with the service tier configured in the
    Project settings. Unless otherwise configured, the Project will use 'default'.
    * If set to 'default', then the requset will be processed with the standard pricing and
    performance for the selected model.
    * If set to '[flex](/docs/guides/flex-processing)' or 'priority', then the request will be
    processed with the corresponding service tier. [Contact
    sales](https://openai.com/contact-sales) to learn more about Priority processing.
    * When not set, the default behavior is 'auto'.

      When the ``service_tier`` parameter is set, the response body will include the
    ``service_tier`` value based on the processing mode actually used to serve the request. This
    response value may be different from the value set in the parameter.
    """

    AUTO = "auto"
    DEFAULT = "default"
    FLEX = "flex"
    SCALE = "scale"
    PRIORITY = "priority"


class ToolChoiceObjectType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Indicates that the model should use a built-in tool to generate a response.
    `Learn more about built-in tools </docs/guides/tools>`_.
    """

    FILE_SEARCH = "file_search"
    FUNCTION = "function"
    COMPUTER = "computer_use_preview"
    WEB_SEARCH = "web_search_preview"
    IMAGE_GENERATION = "image_generation"
    CODE_INTERPRETER = "code_interpreter"
    MCP = "mcp"


class ToolChoiceOptions(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Controls which (if any) tool is called by the model.

    ``none`` means the model will not call any tool and instead generates a message.

    ``auto`` means the model can pick between generating a message or calling one or
    more tools.

    ``required`` means the model must call one or more tools.
    """

    NONE = "none"
    AUTO = "auto"
    REQUIRED = "required"


class ToolType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """A tool that can be used to generate a response."""

    FILE_SEARCH = "file_search"
    FUNCTION = "function"
    COMPUTER_USE_PREVIEW = "computer_use_preview"
    WEB_SEARCH_PREVIEW = "web_search_preview"
    MCP = "mcp"
    CODE_INTERPRETER = "code_interpreter"
    IMAGE_GENERATION = "image_generation"
    LOCAL_SHELL = "local_shell"


class WebSearchActionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of WebSearchActionType."""

    SEARCH = "search"
    OPEN_PAGE = "open_page"
    FIND = "find"



from typing import Dict, List

from langchain_core.messages import AIMessage, AnyMessage, HumanMessage, SystemMessage

from ...models.azureaiagents import models
from ...models.openai import models as openai_models

role_mapping = {
    openai_models.ResponsesMessageRole.USER: HumanMessage,
    openai_models.ResponsesMessageRole.SYSTEM: SystemMessage,
    openai_models.ResponsesMessageRole.ASSISTANT: AIMessage,
    # openai_models.ResponsesMessageRole.DEVELOPER: ToolMessage,
}

item_content_type_mapping = {
    openai_models.ItemContentType.INPUT_TEXT: "text",
    openai_models.ItemContentType.INPUT_AUDIO: "audio",
    openai_models.ItemContentType.INPUT_IMAGE: "image",
    openai_models.ItemContentType.INPUT_FILE: "file",
    openai_models.ItemContentType.OUTPUT_TEXT: "text",
    openai_models.ItemContentType.OUTPUT_AUDIO: "audio",
    # openai_models.ItemContentType.REFUSAL: "refusal",
}


class LangGraphRequestConverter:
    def __init__(self, data: models.CreateResponse, logger):
        self.data: models.CreateResponse = data
        self.logger = logger

    def convert(self) -> dict:
        # Convert the CreateRunRequest input to a format suitable for LangGraph
        langgraph_input = {"messages": []}

        if self.data.instructions:
            langgraph_input["messages"].append(SystemMessage(content=self.data.instructions))

        input = self.data.input
        if isinstance(input, str):
            langgraph_input["messages"].append(HumanMessage(content=input))
        elif isinstance(input, List):
            for inner in input:
                if isinstance(inner, openai_models.ImplicitUserMessage):
                    langgraph_input["messages"].append(self.convert_OpenAIImplicitUserMessage(inner))
                elif isinstance(inner, openai_models.ItemParam):
                    langgraph_input["messages"].append(self.convert_OpenAIItemParam(inner))
                else:
                    raise ValueError(f"Unsupported input type: {type(inner)}")
        return langgraph_input

    def convert_OpenAIImplicitUserMessage(self, input_message: openai_models.ImplicitUserMessage) -> AnyMessage:
        """
        Convert OpenAIImplicitUserMessageContent to a list of message
        """
        if isinstance(input_message.content, str):
            return HumanMessage(content=input_message.content)
        if isinstance(input_message.content, list):
            return HumanMessage(content=self.convert_OpenAIItemContentList(input_message.content))
        raise ValueError(f"Unsupported ImplicitUserMessage content type: {type(input_message.content)}")

    def convert_OpenAIItemContentList(self, content: List[openai_models.ItemContent]) -> List:
        """
        Convert ItemContent to a list format
        """
        result = []
        for item in content:
            if isinstance(item, openai_models.ItemContent):
                result.append(self.convert_OpenAIItemContent(item))
            else:
                raise ValueError(f"Unsupported ItemContent type: {content.type}")
        return result

    def convert_OpenAIItemContent(self, content: openai_models.ItemContent) -> Dict:
        """
        Convert ItemContent to a dict format
        """
        if isinstance(content, openai_models.ItemContent):
            res = content.as_dict()
            res["type"] = item_content_type_mapping.get(content.type, content.type)
            return res
        else:
            raise ValueError(f"Unsupported ItemContent type: {content.type}")

    def convert_OpenAIItemParam(self, item: openai_models.ItemParam) -> List[AnyMessage]:
        """
        Convert OpenAIItemParam to a dict format
        """
        if item.type == openai_models.ItemType.MESSAGE:
            return role_mapping[item.role](content=self.convert_OpenAIItemContentList(item.content))
        else:
            raise ValueError(f"Unsupported OpenAIItemParam type: {item.type}")
